from brian import *
import time

N = 100
N_set = [int(N) for N in randint(100, 8000, size=100)]
#set_global_preferences(useweave=True)

us = usecond
ns = nsecond

def get_predicted_time(N, r):
    theta = 0.02 * N
    T = 1 * second
    dt = 0.1 * ms
    if get_global_preference('useweave'):
        pass
    else:
        overhead_fudge = 80 * us
        wN = 6.4 * us
        wC = 16.4 * us
        wG = 22.3 * us
        aG = 45 * ns
        aG_cachemiss = 138 * ns
        athr = 35.1 * ns
        athr_cachemiss = 90 * ns
        asp = 11 * us
        asyn = 59.3 * ns
    p_lower = (T / dt) * (overhead_fudge + wN + 2 * wC + wG + N * (aG + athr)) + T * r * N * (asp + asyn * theta)
    p_middle = (T / dt) * (overhead_fudge + wN + 2 * wC + wG + N * (aG_cachemiss + athr)) + T * r * N * (asp + asyn * theta)
    p_upper = (T / dt) * (overhead_fudge + wN + 2 * wC + wG + N * (aG_cachemiss + athr_cachemiss)) + T * r * N * (asp + asyn * theta)
    return (p_lower, p_middle, p_upper)

def get_exp(N):
    eqs = '''
    dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
    dge/dt = -ge/(5*ms) : volt
    dgi/dt = -gi/(10*ms) : volt
    '''
    Ne = int(N * 0.8)
    Ni = N - Ne
    P = NeuronGroup(N, model=eqs,
                  threshold= -50 * mV, reset= -60 * mV)
    P.v = -60 * mV + 10 * mV * rand(len(P))
    Pe = P.subgroup(Ne)
    Pi = P.subgroup(Ni)
    Ce = Connection(Pe, P, 'ge')
    Ci = Connection(Pi, P, 'gi')
    Ce.connect_random(Pe, P, 0.02, weight=1.62 * mV)
    Ci.connect_random(Pi, P, 0.02, weight= -9 * mV)
    M = PopulationSpikeCounter(P)
    net = Network(P, Ce, Ci, M)
    net.run(100 * ms)
    M.reinit()
    t = time.time()
    net.run(1 * second)
    t = time.time() - t
    return (float(M.nspikes) / (N * 1. * second), t * second)

#r, t = get_exp(N)
#print 'Rate:', r
#print 'Actual time:', t
#print 'Predicted time:', get_predicted_time(N, r)

rates, t = zip(*[get_exp(N) for N in N_set])
p, pcm_middle, pcm_max = zip(*[get_predicted_time(N, r) for N, r in zip(N_set, rates)])
subplot(121)
plot(N_set, t, '.')
plot(N_set, p, '.')
plot(N_set, pcm_middle, '.')
plot(N_set, pcm_max, '.')
legend(('Actual', 'No cache miss', 'Cache miss on group', 'All cache misses'), 'upper left')
subplot(122)
plot(N_set, rates, '.')
show()



from brian import *
import time

set_global_preferences(useweave=True)

N = 4000
duration = 5 * second

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

Ne = int(N * 0.8)
Ni = N - Ne

P = NeuronGroup(N, model=eqs,
              threshold= -48 * mV)#,reset=-60*mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(Ne)
Pi = P.subgroup(Ni)

Ce = Connection(Pe, P, 'ge')
Ci = Connection(Pi, P, 'gi')
Ce.connect_random(Pe, P, 0.02, weight=1.62 * mV)
Ci.connect_random(Pi, P, 0.02, weight= -9 * mV)

M = PopulationSpikeCounter(P)

#net = MagicNetwork(verbose=False)
net = Network(P, Ce, Ci)
net.run(100 * ms)

t = time.time()
net.run(duration)
print time.time() - t
print M.nspikes
raw_input('Press enter')



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *
import sys, warnings

hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
hrtfset = hrtfdb.load_subject(1002)
hrtf = hrtfset(azim=30, elev=15)

def grpdelay(b, a, nfft=512, whole=False, samplerate=44.1*kHz):
    if not whole:
        nfft *= 2
    w = arange(nfft, dtype=float)/nfft*samplerate
    oa = len(a)-1 # order of a(z)
    oc = oa+len(b)-1 # order of b(z)
    c = convolve(b, a[::-1]) # c(z) = b(z)*a(1/z)*z^-oa
    cr = c*arange(oc+1, dtype=float)
    num = fft(cr, nfft)
    den = fft(c, nfft)
    minmag = 10*sys.float_info.epsilon
    polebins, = (abs(den)<minmag).nonzero()
    if len(polebins):
        warnings.warn('Group delay singular, setting to 0')
        num[b] = 0
        den[b] = 1
    gd = (num/den).real-oa
    if not whole:
        ns = nfft/2
        gd = gd[:ns]
        w = w[:ns]
    return gd, w

gd_l, w = grpdelay(asarray(hrtf.left).flatten(), array([1]), nfft=len(hrtf))
gd_r, w = grpdelay(asarray(hrtf.right).flatten(), array([1]), nfft=len(hrtf))

plot(w, (gd_l-gd_r))#/(44.1*kHz)/usecond)
show()



'''
An example with correlated spike trains
From: Brette, R. (2007). Generation of correlated spike trains.
'''
from brian import *

N = 100
#input = HomogeneousCorrelatedSpikeTrains(N, r=10 * Hz, c=0.1, tauc=10 * ms)

c = .2
nu = linspace(1*Hz, 10*Hz, N)
P = c*dot(nu.reshape((N,1)), nu.reshape((1,N)))/mean(nu**2)
tauc = 5*ms

spikes = mixture_process(nu, P, tauc, 1*second)
#spikes = [(i,t*second) for i,t in spikes]
input = SpikeGeneratorGroup(N, spikes)

S = SpikeMonitor(input)
#S2 = PopulationRateMonitor(input)
#M = StateMonitor(input, 'rate', record=0)
run(1000 * ms)

#subplot(211)
raster_plot(S)
#subplot(212)
#plot(S2.times / ms, S2.smooth_rate(5 * ms))
#plot(M.times / ms, M[0] / Hz)
show()



from brian import *
from scipy import signal, weave, random
from filterbank import Filterbank, RestructureFilterbank
from ..bufferable import Bufferable
try:
    import numexpr
except ImportError:
    numexpr = None
from itertools import izip

# TODO: test all the buffered version of apply_linear_filterbank here
# So far, they seem to more or less work, but probably need more thorough
# testing.

__all__ = ['LinearFilterbank','apply_linear_filterbank']

def apply_linear_filterbank(b, a, x, zi):
    '''
    Parallel version of scipy lfilter command for a bank of n sequences of length 1
    
    In scipy.lfilter, you can apply a filter to multiple sounds at the same time,
    but you can't apply a bank of filters at the same time. This command does
    that. The coeffs b, a must be of shape (n,m,p), x must be of shape (s, n),
    and zi must be of shape (n,m-1,p). Here n is the number of channels in the
    filterbank, m is the order of the filter, p is the number of filters in
    a chain (cascade) to apply (you do first with (:,:,0) then (:,:,1), etc.),
    and s is the size of the buffer segment.
    '''
    alf_cache_b00 = [0]*zi.shape[2]
    alf_cache_a1 = [0]*zi.shape[2]
    alf_cache_b1 = [0]*zi.shape[2]
    alf_cache_zi00 = [0]*zi.shape[2]
    alf_cache_zi0 = [0]*zi.shape[2]
    alf_cache_zi1 = [0]*zi.shape[2]
    for curf in xrange(zi.shape[2]):
        alf_cache_b00[curf] = b[:, 0, curf]
        alf_cache_zi00[curf] = zi[:, 0, curf]
        alf_cache_b1[curf] = b[:, 1:b.shape[1], curf]
        alf_cache_a1[curf] = a[:, 1:b.shape[1], curf]
        alf_cache_zi0[curf] = zi[:, 0:b.shape[1]-1, curf]
        alf_cache_zi1[curf] = zi[:, 1:b.shape[1], curf]
    X = x
    output = empty_like(X)
    num_cascade = zi.shape[2]
    b_loop_size = b.shape[1]-2
    y = zeros(zi.shape[0])
    yr = reshape(y, (1, len(y))).T
    t = zeros(alf_cache_b1[0].shape, order='F')
    t2 = zeros(alf_cache_b1[0].shape, order='F')
    for sample, (x, o) in enumerate(izip(X, output)):
        xr = reshape(x, (1, len(x))).T
        for curf in xrange(num_cascade):
            #y = b[:, 0, curf]*x+zi[:, 0, curf]
            multiply(alf_cache_b00[curf], x, y)
            add(y, alf_cache_zi00[curf], y)
            #zi[:, :i-1, curf] = b[:, 1:i, curf]*xr+zi[:, 1:i, curf]-a[:, 1:i, curf]*yr
            multiply(alf_cache_b1[curf], xr, t)
            add(t, alf_cache_zi1[curf], t)
            multiply(alf_cache_a1[curf], yr, t2)
            subtract(t, t2, alf_cache_zi0[curf])
            u = x
            ur = xr
            x = y
            xr = yr
            y = u
            yr = ur
        #output[sample] = y
        o[:] = x
    return output

if numexpr is not None and False:
    def apply_linear_filterbank(b, a, x, zi):
        X = x
        output = empty_like(X)
        for sample in xrange(X.shape[0]):
            x = X[sample]
            for curf in xrange(zi.shape[2]):
                #y = b[:, 0, curf]*x+zi[:, 0, curf]
                y = numexpr.evaluate('b*x+zi', local_dict={
                        'b':b[:, 0, curf],
                        'x':x,
                        'zi':zi[:, 0, curf]})
                for i in xrange(b.shape[1]-2):
                    #zi[:, i, curf] = b[:, i+1, curf]*x+zi[:, i+1, curf]-a[:, i+1, curf]*y
                    zi[:, i, curf] = numexpr.evaluate('b*x+zi-a*y', local_dict={
                                            'b':b[:, i+1, curf],
                                            'x':x,
                                            'zi':zi[:, i+1, curf],
                                            'a':a[:, i+1, curf],
                                            'y':y})
                i = b.shape[1]-2
                #zi[:, i, curf] = b[:, i+1, curf]*x-a[:, i+1, curf]*y
                zi[:, i, curf] = numexpr.evaluate('b*x-a*y', local_dict={
                                        'b':b[:, i+1, curf],
                                        'x':x,
                                        'a':a[:, i+1, curf],
                                        'y':y})
                x = y
            output[sample] = y
        return output
    

# TODO: accelerate this even more using SWIG instead of weave?
if get_global_preference('useweave'):
    _cpp_compiler = get_global_preference('weavecompiler')
    _extra_compile_args = ['-O3']
    if _cpp_compiler=='gcc':
        _extra_compile_args += get_global_preference('gcc_options')
    _old_apply_linear_filterbank = apply_linear_filterbank
    
    def apply_linear_filterbank(b, a, x, zi):
        if zi.shape[2]>1:
            # we need to do this so as not to alter the values in x in the C code below
            # but if zi.shape[2] is 1 there is only one filter in the chain and the
            # copy operation at the end of the C code will never happen.
            x = array(x, copy=True)
        y = empty_like(x)
        n, m, p = b.shape
        n1, m1, p1 = a.shape
        numsamples = x.shape[0]
        if n1!=n or m1!=m or p1!=p or x.shape!=(numsamples, n) or zi.shape!=(n, m, p):
            raise ValueError('Data has wrong shape.')
        if numsamples>1 and not x.flags['C_CONTIGUOUS']:
            raise ValueError('Input data must be C_CONTIGUOUS')
        if not b.flags['F_CONTIGUOUS'] or not a.flags['F_CONTIGUOUS'] or not zi.flags['F_CONTIGUOUS']:
            raise ValueError('Filter parameters must be F_CONTIGUOUS')
        code = '''
        #define X(s,i) x[(s)*n+(i)]
        #define Y(s,i) y[(s)*n+(i)]
        #define A(i,j,k) a[(i)+(j)*n+(k)*n*m]
        #define B(i,j,k) b[(i)+(j)*n+(k)*n*m]
        #define Zi(i,j,k) zi[(i)+(j)*n+(k)*n*(m-1)]
        for(int s=0; s<numsamples; s++)
        {
            for(int k=0; k<p; k++)
            {
                for(int j=0; j<n; j++)
                             Y(s,j) =   B(j,0,k)*X(s,j) + Zi(j,0,k);
                for(int i=0; i<m-2; i++)
                    for(int j=0;j<n;j++)
                        Zi(j,i,k) = B(j,i+1,k)*X(s,j) + Zi(j,i+1,k) - A(j,i+1,k)*Y(s,j);
                for(int j=0; j<n; j++)
                      Zi(j,m-2,k) = B(j,m-1,k)*X(s,j)               - A(j,m-1,k)*Y(s,j);
                if(k<p-1)
                    for(int j=0; j<n; j++)
                        X(s,j) = Y(s,j);
            }
        }
        '''
        if 0: # this version is used for comparing without vectorising over frequency
            code = '''
            #define X(s,i) x[(s)*n+(i)]
            #define Y(s,i) y[(s)*n+(i)]
            #define A(i,j,k) a[(i)+(j)*n+(k)*n*m]
            #define B(i,j,k) b[(i)+(j)*n+(k)*n*m]
            #define Zi(i,j,k) zi[(i)+(j)*n+(k)*n*(m-1)]
            for(int j=0; j<n; j++)
            {
                for(int k=0; k<p; k++)
                {
                    for(int s=0; s<numsamples; s++)
                    {
                    
                        Y(s,j) =   B(j,0,k)*X(s,j) + Zi(j,0,k);
                        for(int i=0; i<m-2; i++)
                            Zi(j,i,k) = B(j,i+1,k)*X(s,j) + Zi(j,i+1,k) - A(j,i+1,k)*Y(s,j);                    
                        Zi(j,m-2,k) = B(j,m-1,k)*X(s,j)               - A(j,m-1,k)*Y(s,j);
                        if(k<p-1)
                            X(s,j) = Y(s,j);
                    }
                }
            }
            '''
        weave.inline(code, ['b', 'a', 'x', 'zi', 'y', 'n', 'm', 'p', 'numsamples'],
                     compiler=_cpp_compiler,
                     extra_compile_args=_extra_compile_args)
        return y
    apply_linear_filterbank.__doc__ = _old_apply_linear_filterbank.__doc__


class LinearFilterbank(Filterbank):
    '''
    Generalised linear filterbank
    
    Initialisation arguments:

    ``source``
        The input to the filterbank, must have the same number of channels or
        just a single channel. In the latter case, the channels will be
        replicated.
    ``b``, ``a``
        The coeffs b, a must be of shape ``(nchannels, m)`` or
        ``(nchannels, m, p)``. Here ``m`` is
        the order of the filters, and ``p`` is the number of filters in a
        chain (first you apply ``[:, :, 0]``, then ``[:, :, 1]``, etc.).
    
    The filter parameters are stored in the modifiable attributes ``filt_b``,
    ``filt_a`` and ``filt_state`` (the variable ``z`` in the section below).
    
    Has one method:
    
    .. automethod:: decascade
    
    **Notes**
    
    These notes adapted from scipy's :func:`~scipy.signal.lfilter` function.
    
    The filterbank is implemented as a direct II transposed structure.
    This means that for a single channel and element of the filter cascade,
    the output y for an input x is defined by::

        a[0]*y[m] = b[0]*x[m] + b[1]*x[m-1] + ... + b[m]*x[0]
                              - a[1]*y[m-1] - ... - a[m]*y[0]

    using the following difference equations::

        y[i] = b[0]*x[i] + z[0,i-1]
        z[0,i] = b[1]*x[i] + z[1,i-1] - a[1]*y[i]
        ...
        z[m-3,i] = b[m-2]*x[i] + z[m-2,i-1] - a[m-2]*y[i]
        z[m-2,i] = b[m-1]*x[i] - a[m-1]*y[i]

    where i is the output sample number.

    The rational transfer function describing this filter in the
    z-transform domain is::
    
                                -1              -nb
                    b[0] + b[1]z  + ... + b[m] z
            Y(z) = --------------------------------- X(z)
                                -1              -na
                    a[0] + a[1]z  + ... + a[m] z
        
    '''
    def __init__(self, source, b, a):
        # Automatically duplicate mono input to fit the desired output shape
        if b.shape[0]!=source.nchannels:
            if source.nchannels!=1:
                raise ValueError('Can only automatically duplicate source channels for mono sources, use RestructureFilterbank.')
            source = RestructureFilterbank(source, b.shape[0])
        Filterbank.__init__(self, source)
        # Weave version of filtering requires Fortran ordering of filter params
        if len(b.shape)==2 and len(a.shape)==2:
            b = reshape(b, b.shape+(1,))
            a = reshape(a, a.shape+(1,))
        self.filt_b = array(b, order='F')
        self.filt_a = array(a, order='F')
        self.filt_state = zeros((b.shape[0], b.shape[1], b.shape[2]), order='F')

    def reset(self):
        self.buffer_init()
        
    def buffer_init(self):
        Filterbank.buffer_init(self)
        self.filt_state[:] = 0
    
    def buffer_apply(self, input):
        
        return apply_linear_filterbank(self.filt_b, self.filt_a, input,
                                       self.filt_state)
        
    def decascade(self, ncascade=1):
        '''
        Reduces cascades of low order filters into smaller cascades of high order filters.
        
        ``ncascade`` is the number of cascaded filters to use, which should be
        a divisor of the original number.
        
        Note that higher order filters are often numerically unstable.
        '''
        n, m, p = self.filt_b.shape
        if p%ncascade!=0:
            raise ValueError('Number of cascades must be a divisor of original number of cascaded filters.')
        b = zeros((n, (m-1)*(p/ncascade)+1, ncascade))
        a = zeros((n, (m-1)*(p/ncascade)+1, ncascade))
        for i in xrange(n):
            for k in xrange(ncascade):
                bp = ones(1)
                ap = ones(1)
                for j in xrange(k*(p/ncascade), (k+1)*(p/ncascade)):
                    bp = polymul(bp, self.filt_b[i, ::-1, j])
                    ap = polymul(ap, self.filt_a[i, ::-1, j])
                bp = bp[::-1]
                ap = ap[::-1]
                a0 = ap[0]
                ap /= a0
                bp /= a0
                b[i, :len(bp), k] = bp
                a[i, :len(ap), k] = ap
        self.filt_b = array(b, order='F')
        self.filt_a = array(a, order='F')
        self.filt_state = zeros((b.shape[0], b.shape[1], b.shape[2]), order='F')
                
# Use the GPU version if available
try:
    if get_global_preference('brianhears_usegpu'):
        import pycuda
        from gpulinearfilterbank import LinearFilterbank
        use_gpu = True
    else:
        use_gpu = False
except ImportError:
    use_gpu = False



'''
NOTE: you need a .pypirc file to do this, you may need to set the
HOME env to where it is saved. Also note that any spaces in the
filename of HOME will cause it to not work, so use old style 8.3
equivalent name.

Also note that manifest.in may not work right with this?
'''
import os
os.chdir('../../../.') # work from Brian's root
os.system('setup.py register')
os.system('setup.py sdist bdist_wininst upload')



from brian import *

# long run vals
Nvals = [1000, 2000, 4000, 8000, 16000, 32000]
N_varycon = 1000
N_varywe = 4000
Nsynvals = [80., 200., 500., 1000.]
wevals = [1.62 * mV, 3 * mV, 6 * mV, 9 * mV]
duration = 2.5 * second
repeats = 10
best = 7
cuba_opts = {'connections':True}
# debug run vals
#Nvals = [1000, 2000]
#duration = 2.5*second
#repeats = 3
#best = 2



#!/usr/bin/env python
from brian import *
from brian.library.IF import *

T = 3000 * ms

Ni = 200
Ne = 800
N = Ni + Ne
conn_prob = 0.1
exc_weigth0 = 6 * mV
inh_weigth0 = -5 * mV
max_delay = 20 * ms
vm0 = -65 * mV
w0 = 0.2 * vm0 / ms
Vt = -30 * mV
input_freq = 20 * Hz
input_curr = 20 * mV

eqs_e = Izhikevich(a=0.02 / ms, b=0.2 / ms)
rst_e = AdaptiveReset(Vr= -75 * mV, b=8.0 * nA)
Me = Model(model=eqs_e, threshold=Vt, reset=rst_e, max_delay=max_delay)

eqs_i = Izhikevich(a=0.1 / ms, b=0.2 / ms)
rst_i = AdaptiveReset(Vr= -75 * mV, b=2.0 * nA)
Mi = Model(model=eqs_i, threshold=Vt, reset=rst_i, max_delay=max_delay)

Gi = Ni * Mi
Gi.vm = vm0
Gi.w = w0

Ge = Ni * Mi
Ge.vm = vm0
Ge.w = w0

def nextspike():
    interval = 1 / input_freq
    nexttime = interval
    while True:
        yield (0, nexttime)
        nexttime += interval

# random thalamic input
input = SpikeGeneratorGroup(1, nextspike())
Ice = Connection(input, Ge)
Ici = Connection(input, Gi)

# uniform distribution of exc. synaptic delays
Cee = DelayConnection(Ge, Ge, max_delay=20 * ms)
Cee.connect_random(Ge, Ge, conn_prob / 2, exc_weigth0, delay=(0 * ms, 20 * ms))
Cei = DelayConnection(Ge, Gi, max_delay=20 * ms)
Cei.connect_random(Ge, Gi, conn_prob / 2, exc_weigth0, delay=(0 * ms, 20 * ms))

Cie = DelayConnection(Gi, Ge, max_delay=1 * ms)
# all inhibitory delays are 1 ms
Cie.connect_random(Gi, Ge, conn_prob, inh_weigth0, delay=1 * ms)

#STDP
gmax = 0.010
tau_pre = 20 * ms
tau_post = 20 * ms
dA_pre = gmax * .005
dA_post = -dA_pre * 1.05

eqs_stdp = """
dA_pre/dt  = -A_pre/tau_pre   : 1
dA_post/dt = -A_post/tau_post : 1
"""
stdp_ee = STDP(Cee, eqs=eqs_stdp, pre='''A_pre+=dA_pre; w+=A_post''',
            post='''A_post+=dA_post; w+=A_pre''', wmax=gmax)
stdp_ei = STDP(Cei, eqs=eqs_stdp, pre='''A_pre+=dA_pre; w+=A_post''',
            post='''A_post+=dA_post; w+=A_pre''', wmax=gmax)
stdp_ie = STDP(Cie, eqs=eqs_stdp, pre='''A_pre+=dA_pre; w+=A_post''',
            post='''A_post+=dA_post; w+=A_pre''', wmax=gmax)

Se = SpikeMonitor(Ge)
Si = SpikeMonitor(Gi)

run(T)
raster_plot(Se, Si)

show()



from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
                    reshape, sum, log, Monitor, NetworkOperation, defaultclock, linspace, vstack, \
                    arange, sort_spikes, rint, SpikeMonitor, Connection, StateMonitor
from brian.tools.statistics import firing_rate, get_gamma_factor
try:
    from playdoh import *
except Exception, e:
    print e
    raise ImportError("Playdoh must be installed (https://code.google.com/p/playdoh/)")

try:
    import pycuda
    from gpu_modelfitting import GPUModelFitting
    can_use_gpu = True
except ImportError:
    can_use_gpu = False
from brian.experimental.codegen.integration_schemes import *
from criteria import *
import sys, cPickle




class DataTransformer(object):
    """
    Transform spike, input and trace data from user-friendly data structures,
    like 2 dimensional arrays or lists of spikes, into algorithm- and GPU-friendly structures,
    i.e. inline vectors (1 dimensional) easily parallelizable.
    """
    def __init__(self, neurons, input, spikes = None, traces = None, dt = defaultclock.dt,
                 slices = 1, overlap = 0*ms, groups = 1):
        self.neurons = neurons # number of particles on the node
        self.input = input # a IxT array
        self.spikes = spikes # a list of spikes [(i,t)...]
        self.traces = traces # a KxT array
        self.slices = slices
        self.overlap = overlap
        self.groups = groups
        self.dt = dt
        # ensure 2 dimensions
        if self.input.ndim == 1:
            self.input = self.input.reshape((1,-1))
        if self.traces is not None:
            if self.traces.ndim == 1:
                self.traces = self.traces.reshape((1,-1))
        self.inputs_count = self.input.shape[0]
        self.T = self.input.shape[1] # number of steps
        self.duration = self.T*self.dt
        self.subpopsize = self.neurons/self.groups # number of neurons per group: nodesize/groups
        self.input = self.input[:,0:self.slices * (self.T / self.slices)] # makes sure that len(input) is a multiple of slices
        self.sliced_steps = self.T / self.slices # timesteps per slice
        self.overlap_steps = int(self.overlap / self.dt) # timesteps during the overlap
        self.total_steps = self.sliced_steps + self.overlap_steps # total number of timesteps
        self.sliced_duration = self.overlap + self.duration / self.slices # duration of the vectorized simulation
        self.N = self.neurons * self.slices # TOTAL number of neurons on this node
        self.input = hstack((zeros((self.inputs_count, self.overlap_steps)), self.input)) # add zeros at the beginning because there is no overlap from the previous slice
        
    def slice_spikes(self, spikes):
        # from standard structure to standard structure
        sliced_spikes = []
        slice_length = self.sliced_steps*self.dt
        for (i,t) in spikes:
            slice = int(t/slice_length)
            newt = self.overlap + (t % slice_length)*second
            newi = i + self.groups*slice
            # discard unreachable spikes
            if newi >= (self.slices*self.groups): continue
            sliced_spikes.append((newi, newt)) 
        sliced_spikes = sort_spikes(sliced_spikes)
        return sliced_spikes

    def slice_traces(self, traces):
        # from standard structure to standard structure
        k = traces.shape[0]
        sliced_traces = zeros((k*self.slices, self.total_steps))
        for slice in xrange(self.slices):
            i0 = slice*k
            i1 = (slice+1)*k
            j0 = slice*self.sliced_steps
            j1 = (slice+1)*self.sliced_steps
            sliced_traces[i0:i1,self.overlap_steps:] = traces[:,j0:j1]
            if slice>0:
                sliced_traces[i0:i1,:self.overlap_steps] = traces[:,j0-self.overlap_steps:j0]
        return sliced_traces

    def transform_spikes(self, spikes):
        # from standard structure to inline structure
        i, t = zip(*spikes)
        i = array(i)
        t = array(t)
        alls = []
        n = 0
        pointers = []
        model_target = []
        for j in xrange(self.groups):
            s = sort(t[i == j])
            s = hstack((-1 * second, s, self.duration + 1 * second))
            model_target.extend([j] * self.subpopsize)
            alls.append(s)
            pointers.append(n)
            n += len(s)
        pointers = array(pointers, dtype=int)
        model_target = array(hstack(model_target), dtype=int)
        spikes_inline = hstack(alls)
        spikes_offset = pointers[model_target]
        return spikes_inline, spikes_offset
    
    def transform_traces(self, traces):
        # from standard structure to inline structure
        K, T = traces.shape
        traces_inline = traces.flatten()
        traces_offset = array(kron(arange(K), T*ones(self.subpopsize)), dtype=int)
        return traces_inline, traces_offset




class Simulator(object):
    def __init__(self, model, reset, threshold, 
                 inputs, input_var = 'I', dt = defaultclock.dt,
                 refractory = 0*ms, max_refractory = None,
                 spikes = None, traces = None,
                 groups = 1,
                 slices = 1, overlap = 0*second,
                 onset = 0*second,
                 neurons = 1000, # = nodesize = number of neurons on this node = total number of neurons/slices
                 initial_values = None,
                 unit_type = 'CPU',
                 stepsize = 100*ms,
                 precision = 'double',
                 criterion = None,
                 statemonitor_var=None,
                 method = 'Euler'
                 ):
        self.model = model
        self.reset = reset
        self.threshold = threshold
        self.inputs = inputs
        self.input_var = input_var
        self.dt = dt
        self.refractory = refractory
        self.max_refractory = max_refractory
        self.spikes = spikes
        self.traces = traces
        self.initial_values = initial_values 
        self.groups = groups
        self.slices = slices
        self.overlap = overlap
        self.onset = onset
        self.neurons = neurons
        self.unit_type = unit_type
        self.statemonitor_var = statemonitor_var
        self.stepsize = stepsize
        self.precision = precision
        self.criterion = criterion
        self.method = method
        self.use_gpu = self.unit_type=='GPU'
        
        if self.statemonitor_var is not None:
            self.statemonitor_values = zeros(self.neurons)
        
        self.initialize_neurongroup()
        self.transform_data()
        self.inject_input()
        self.initialize_criterion(delays=zeros(self.neurons))
        
        if self.use_gpu:
            self.initialize_gpu()
            
    def initialize_neurongroup(self):
        # Add 'refractory' parameter on the CPU only
        if not self.use_gpu:
            if self.max_refractory is not None:
                refractory = 'refractory'
                self.model.add_param('refractory', second)
            else:
                refractory = self.refractory
        else:
            if self.max_refractory is not None:
                refractory = 0*ms
            else:
                refractory = self.refractory
        
        # Must recompile the Equations : the functions are not transfered after pickling/unpickling
        self.model.compile_functions()
        
        self.group = NeuronGroup(self.neurons,
                                 model=self.model,
                                 reset=self.reset,
                                 threshold=self.threshold,
                                 refractory=refractory,
                                 max_refractory = self.max_refractory,
                                 method = self.method,
                                 clock=Clock(dt=self.dt))
        
        if self.initial_values is not None:
            for param, value in self.initial_values.iteritems():
                self.group.state(param)[:] = value
    
    def initialize_gpu(self):
            # Select integration scheme according to method
            if self.method == 'Euler': scheme = euler_scheme
            elif self.method == 'RK': scheme = rk2_scheme
            elif self.method == 'exponential_Euler': scheme = exp_euler_scheme
            else: raise Exception("The numerical integration method is not valid")
            
            self.mf = GPUModelFitting(self.group, self.model, self.criterion_object,
                                      self.input_var,
                                      self.onset, 
                                      statemonitor_var = self.statemonitor_var,
                                      duration = self.sliced_duration,
                                      precision=self.precision, scheme=scheme)
    
    def transform_data(self):
        self.transformer = DataTransformer(self.neurons,
                                           self.inputs,
                                           spikes = self.spikes, 
                                           traces = self.traces,
                                           dt = self.dt,
                                           slices = self.slices,
                                           overlap = self.overlap, 
                                           groups = self.groups)
        self.total_steps = self.transformer.total_steps
        self.sliced_duration = self.transformer.sliced_duration
        
        self.sliced_inputs = self.transformer.slice_traces(self.inputs)
        self.inputs_inline, self.inputs_offset = self.transformer.transform_traces(self.sliced_inputs)
        
        if self.traces is not None:
            self.sliced_traces = self.transformer.slice_traces(self.traces)
            self.traces_inline, self.traces_offset = self.transformer.transform_traces(self.sliced_traces)
        else:
            self.sliced_traces, self.traces_inline, self.traces_offset = None, None, None
        
        if self.spikes is not None:
            self.sliced_spikes = self.transformer.slice_spikes(self.spikes)
            self.spikes_inline, self.spikes_offset = self.transformer.transform_spikes(self.sliced_spikes)
        else:
            self.sliced_spikes, self.spikes_inline, self.spikes_offset = None, None, None
    
    def inject_input(self):
        # Injects current in consecutive subgroups, where I_offset have the same value
        # on successive intervals
        I_offset = self.inputs_offset
        k = -1
        for i in hstack((nonzero(diff(I_offset))[0], len(I_offset) - 1)):
            I_offset_subgroup_value = I_offset[i]
            I_offset_subgroup_length = i - k
            sliced_subgroup = self.group.subgroup(I_offset_subgroup_length)
            input_sliced_values = self.inputs_inline[I_offset_subgroup_value:I_offset_subgroup_value + self.total_steps]
            sliced_subgroup.set_var_by_array(self.input_var, TimedArray(input_sliced_values, clock=self.group.clock))
            k = i
    
    def initialize_criterion(self, **criterion_params):
        # general criterion parameters
        params = dict(group=self.group, traces=self.sliced_traces, spikes=self.sliced_spikes, 
                      targets_count=self.groups*self.slices, duration=self.sliced_duration, onset=self.onset, 
                      spikes_inline=self.spikes_inline, spikes_offset=self.spikes_offset,
                      traces_inline=self.traces_inline, traces_offset=self.traces_offset)
        for key,val in criterion_params.iteritems():
            params[key] = val
        criterion_name = self.criterion.__class__.__name__
        
        # criterion-specific parameters
        if criterion_name == 'GammaFactor':
            params['delta'] = self.criterion.delta
            params['coincidence_count_algorithm'] = self.criterion.coincidence_count_algorithm
            self.criterion_object = GammaFactorCriterion(**params)
            
        if criterion_name == 'LpError':
            params['p'] = self.criterion.p
            params['varname'] = self.criterion.varname
            self.criterion_object = LpErrorCriterion(**params)
    
    def update_neurongroup(self, **param_values):
        """
        Inject fitting parameters into the NeuronGroup
        """
        # Sets the parameter values in the NeuronGroup object
        self.group.reinit()
        for param, value in param_values.iteritems():
            self.group.state(param)[:] = kron(value, ones(self.slices)) # kron param_values if slicing
        
        # Reinitializes the model variables
        if self.initial_values is not None:
            for param, value in self.initial_values.iteritems():
                self.group.state(param)[:] = value
    
    def combine_sliced_values(self, values):
        if type(values) is tuple:
            combined_values = tuple([sum(reshape(v, (self.slices, -1)), axis=0) for v in values])
        else:
            combined_values = sum(reshape(values, (self.slices, -1)), axis=0)
        return combined_values
    
    def run(self, **param_values):
        delays = param_values.pop('delays', zeros(self.neurons))
        refractory = param_values.pop('refractory', zeros(self.neurons))
        
        self.update_neurongroup(**param_values)

        # repeat spike delays and refractory to take slices into account
        delays = kron(delays, ones(self.slices))
        refractory = kron(refractory, ones(self.slices))
        
        # TODO: add here parameters to criterion_params if a criterion must use some parameters
        criterion_params = dict(delays=delays)

        self.update_neurongroup(**param_values)
        self.initialize_criterion(**criterion_params)
        
        if self.use_gpu:
            # Reinitializes the simulation object
            self.mf.reinit_vars(self.criterion_object,
                                self.inputs_inline, self.inputs_offset,
                                self.spikes_inline, self.spikes_offset,
                                self.traces_inline, self.traces_offset,
                                delays, refractory
                                )
            # LAUNCHES the simulation on the GPU
            self.mf.launch(self.sliced_duration, self.stepsize)
            # Synchronize the GPU values with a call to gpuarray.get()
            self.criterion_object.update_gpu_values()
        else:
            # set the refractory period
            if self.max_refractory is not None:
                self.group.refractory = refractory
            # Launch the simulation on the CPU
            self.group.clock.reinit()
            net = Network(self.group, self.criterion_object)
            if self.statemonitor_var is not None:
                self.statemonitor = StateMonitor(self.group, self.statemonitor_var, record=True)
                net.add(self.statemonitor)
            net.run(self.sliced_duration)
        
        sliced_values = self.criterion_object.get_values()
        combined_values = self.combine_sliced_values(sliced_values)
        values = self.criterion_object.normalize(combined_values)
        return values

    def get_statemonitor_values(self):
        if not self.use_gpu:
            return self.statemonitor.values
        else:
            return self.mf.get_statemonitor_values()




def simulate( model, reset = None, threshold = None, 
                 input = None, input_var = 'I', dt = defaultclock.dt,
                 refractory = 0*ms, max_refractory = None,
                 data = None,
                 groups = 1,
                 slices = 1,
                 overlap = 0*second,
                 neurons = 1,
                 initial_values = None,
                 use_gpu = False,
                 stepsize = 100*ms,
                 precision = 'double',
                 criterion = None,
                 record = None,
                 method = 'Euler',
                 **params):
    
    unit_type = 'CPU'
    if use_gpu: unit_type = 'GPU'
    
    for param in params.keys():
        if (param not in model._diffeq_names) and (param != 'delays'):
            raise Exception("Parameter %s must be defined as a parameter in the model" % param)
    
    if criterion is None:
        criterion = GammaFactor()
    
    data = array(data)
    if criterion.type == 'spikes':
        # Make sure that 'data' is a N*2-array
        if data.ndim == 1:
            data = concatenate((zeros((len(data), 1)), data.reshape((-1, 1))), axis=1)
        spikes = data
        traces = None
        groups = int(array(data)[:, 0].max() + 1) # number of target trains
    elif criterion.type == 'trace':
        if data.ndim == 1:
            data = data.reshape((1,-1))
        spikes = None
        traces = data
        groups = data.shape[0]
    elif criterion.type == 'both':
        # TODO
        log_warn("Not implemented yet")
        pass
    inputs = input
    if inputs.ndim==1:
        inputs = inputs.reshape((1,-1))

    # dt must be set
    if dt is None:
        raise Exception('dt (sampling frequency of the input) must be set')

    # default overlap when no time slicing
    if slices == 1:
        overlap = 0*ms

    # check numerical integration method
    if use_gpu and method not in ['Euler', 'RK', 'exponential_Euler']:
        raise Exception("The method can only be 'Euler', 'RK', or 'exponential_Euler' when using the GPU") 
    if method not in ['Euler', 'RK', 'exponential_Euler', 'linear', 'nonlinear']:
        raise Exception("The method can only be 'Euler', 'RK', 'exponential_Euler', 'linear', or 'nonlinear'")

    # determines whether optimization over refractoriness or not
    if type(refractory) is tuple or type(refractory) is list:
        params['refractory'] = refractory
        max_refractory = refractory[-1]
    else:
        max_refractory = None
    
    # Initialize GPU
    if use_gpu:
        set_gpu_device(0)
    
    simulator = Simulator(model, reset, threshold, 
                         inputs,
                         input_var = input_var,
                         dt = dt,
                         refractory = refractory,
                         max_refractory = max_refractory,
                         spikes = spikes,
                         traces = traces,
                         groups = groups,
                         slices = slices,
                         overlap = overlap,
                         onset = overlap,
                         neurons = neurons,
                         initial_values = initial_values,
                         unit_type = unit_type,
                         stepsize = stepsize,
                         precision = precision,
                         criterion = criterion,
                         statemonitor_var = record,
                         method = method)
    criterion_values = simulator.run(**params)
    if record is not None:
        record_values = simulator.get_statemonitor_values()
        return criterion_values, record_values
    else:
        return criterion_values




if __name__ == '__main__':
    
    from brian import loadtxt, ms, savetxt, loadtxt, Equations, NeuronGroup, run, SpikeMonitor,\
         StateMonitor, Network
    from pylab import *
    
    equations = Equations('''
        dV/dt=(R*I-V)/tau : 1
        I : 1
        R : 1
        tau : second
    ''')
    input = loadtxt('current.txt')
    
    trace = loadtxt('trace_artificial.txt')
    spikes= loadtxt('spikes_artificial.txt')
    
    # GAMMA FACTOR
    criterion = GammaFactor(delta=2*ms)
    data = spikes
#    data[:,1] += 50*ms
    
    # LP ERROR
    criterion = LpError(p=2, varname='V')
    data = trace
    
    # SIMULATE, EVALUATE CRITERION AND RECORD TRACES ON GPU
    neurons = 5
    R = [3e9]*neurons
    tau = linspace(25*ms, 30*ms, neurons)
    criterion_values, record_values = simulate(model = equations,
                                reset = 0,
                                threshold = 1,
                                data = data,
                                input = input,
                                use_gpu = True,
                                dt = .1*ms,
                                criterion = criterion,
                                record = 'V',
                                neurons = neurons,
                                R = R,
                                tau = tau,
                                )
    print criterion_values
    
    plot(trace)
    plot(record_values.transpose())
    show()
    
    


'''
Tutorial 1f: Recording spikes
*****************************

In the previous part of the tutorial, we defined a network with
not entirely trivial behaviour, and printed the number of spikes.
In this part, we'll record every spike that the network generates
and display a raster plot of them. We start as before:
'''
from brian import *

tau = 20 * msecond        # membrane time constant
Vt = -50 * mvolt          # spike threshold
Vr = -60 * mvolt          # reset value
El = -49 * mvolt          # resting potential (same as the reset)
psp = 0.5 * mvolt         # postsynaptic potential size

G = NeuronGroup(N=40, model='dV/dt = -(V-El)/tau : volt',
              threshold=Vt, reset=Vr)

C = Connection(G, G)
C.connect_random(sparseness=0.1, weight=psp)

M = SpikeMonitor(G)

G.V = Vr + rand(40) * (Vt - Vr)

run(1 * second)

print M.nspikes
'''
Having run the network, we simply use the :func:`raster_plot` function
provided by Brian. After creating plots, we have to use the
``show()`` function to display them. This function is from the
PyLab module that Brian uses for its built in plotting
routines.
'''
raster_plot()
show()
'''

.. image:: images/tutorials/1f.jpg

As you can see, despite having introduced some randomness into
our network, the output is very regular indeed. In the next part
we introduce one more way to plot the output of a network.
'''



'''
Simple implementation of an event scheduling scheme for Brian.

The idea is that a spike schedules a future event V[i] += val for i an index
and val a float value. Below this idea is used to implement dendritic delays
in an alternative way to DelayConnection. You could implement axonal delays by
using V[i] += W[j] for i and j both indices (here j would be a pointer to an
appropriate entry in a weight matrix W, and i an appropriate neuron state
variable index).
'''

from brian import *
from scipy import weave
import numpy
from itertools import izip

N = 10
max_events_per_timestep = 100000

G = NeuronGroup(1, 'V:1', reset=0, threshold=0.5)
H = NeuronGroup(N, 'V:1', reset=0, threshold=0.5)


class ScheduledEventDelayConnection(DelayConnection):
    def __init__(self, *args, **kwds):
        DelayConnection.__init__(self, *args, **kwds)
        self.scheduled_events_ptr = numpy.zeros((self._max_delay, max_events_per_timestep), dtype=int)
        self.scheduled_events_val = numpy.zeros((self._max_delay, max_events_per_timestep), dtype=float)
        self.num_scheduled_events = numpy.zeros(self._max_delay, dtype=int)
        self.scheduled_events_index = 0
        @network_operation(when='after_connections') # for the moment, don't worry about clock details
        def process_scheduled_events():
            num_scheduled_events = int(self.num_scheduled_events[self.scheduled_events_index])
            if num_scheduled_events:
                ptr = self.scheduled_events_ptr[self.scheduled_events_index, :num_scheduled_events]
                val = self.scheduled_events_val[self.scheduled_events_index, :num_scheduled_events]
                sv = self.target._S[self.nstate]
                #sv[ptr] += val
                I = argsort(ptr)
                ptrs = ptr[I]
                vals = val[I]
                J = hstack((True, (ptrs[1:]!=ptrs[:-1]))).nonzero()[0]
                sv[ptrs[J]] += add.reduceat(vals, J)
                # with a bit of work, this can be made to work for any ufunc using
                # the .reduceat method of ufuncs after sorting ptr, etc. Still doesn't work
                # in the general case of any Python expression though.
                self.num_scheduled_events[self.scheduled_events_index] = 0
            self.scheduled_events_index = (self.scheduled_events_index + 1) % self._max_delay
        self.contained_objects.append(process_scheduled_events)
        self.constant_delays = True # set to True to assume that delays will not change over time
        self.constant_weights = True
        self.need_to_precompute_sorted_delays = self.constant_delays
        for f in self.contained_objects:
            if f.__name__=='delayed_propagate':
                self.contained_objects.remove(f)
                break

    def propagate(self, spikes):
        if not self.iscompressed:
            self.compress()
        if self.need_to_precompute_sorted_delays:
            self.sort_indices = []
            self.int_delay = []
            self.target_indices = []
            self.sorted_Wrow = []
            self.Jp = []
            self.event_index_offset = []
            self.added_events = []
            for i in xrange(len(self.source)):
                dvecrow = self.delayvec[i, :]
                int_delay = array(self._invtargetdt * dvecrow, dtype=int)
                sort_indices = argsort(int_delay)
                int_delay = int_delay[sort_indices]
                J = int_delay[1:] != int_delay[:-1]
                K = int_delay[1:] == int_delay[:-1]
                A = hstack((0, cumsum(array(J, dtype=int))))
                B = hstack((0, cumsum(array(K, dtype=int))))
                BJ = hstack((0, B[J]))
                event_index_offset = B - BJ[A]
                Jp = hstack((J, True))
                added_events = event_index_offset[Jp] + 1
                self.sort_indices.append(sort_indices)
                self.int_delay.append(int_delay)
                self.event_index_offset.append(event_index_offset)
                self.Jp.append(Jp)
                self.target_indices.append(dvecrow.ind[sort_indices])
                self.added_events.append(added_events)
                if self.constant_weights:
                    self.sorted_Wrow.append(self.W[i, :][sort_indices])
            self.need_to_precompute_sorted_delays = False
        if len(spikes):
            if False:
                # C code version
                nspikes = len(spikes)
                W = asarray(self.W)
                n, m = W.shape
                numevents = self.num_scheduled_events
                ptr, val = self.scheduled_events_ptr, self.scheduled_events_val
                index = self.scheduled_events_index
                idt = self._invtargetdt
                md = self._max_delay
                dvec = asarray(self.delayvec)
                sv = self.target._S[self.nstate]
                code = '''
                for(int i=0; i<nspikes; i++)
                {
                    int k = spikes(i);
                    for(int j=0; j<m; j++)
                    {
                        int queue_index = (index+(int)(idt*dvec(k, j)))%md;
                        int event_index = numevents(queue_index)++;
                        ptr(queue_index, event_index) = j;
                        val(queue_index, event_index) = W(k, j);
                    }
                }
                '''
                weave.inline(code, ['numevents', 'ptr', 'val', 'nspikes', 'n', 'm', 'index', 'idt',
                                    'md', 'dvec', 'sv', 'spikes', 'W'],
                             type_converters=weave.converters.blitz,
                             compiler='gcc')
            else:
                # Python code only version
                ptr, val = self.scheduled_events_ptr, self.scheduled_events_val
                #W = asarray(self.W)
                Wrows = self.W.get_rows(spikes)
                dvecrows = self.delayvec.get_rows(spikes)
                num_scheduled_events = self.num_scheduled_events
                sparse = isinstance(dvecrows[0], SparseConnectionVector)
                _max_delay = self._max_delay
                scheduled_events_index = self.scheduled_events_index
                precomp = self.constant_delays
                precomp_weights = self.constant_weights
                for i, dvecrow, Wrow in izip(spikes, dvecrows, Wrows):
                    dvecrowarr = asarray(dvecrow)
                    if precomp:
                        int_delay = self.int_delay[i]
                        sort_indices = self.sort_indices[i]
                        event_index_offset = self.event_index_offset[i]
                        Jp = self.Jp[i]
                        added_events = self.added_events[i]
                    else:
                        int_delay = array(self._invtargetdt * dvecrowarr, dtype=int)
                        sort_indices = argsort(int_delay)
                        int_delay = int_delay[sort_indices]
                        J = int_delay[1:] != int_delay[:-1]
                        K = int_delay[1:] == int_delay[:-1]
                        A = hstack((0, cumsum(array(J, dtype=int))))
                        B = hstack((0, cumsum(array(K, dtype=int))))
                        BJ = hstack((0, B[J]))
                        event_index_offset = B - BJ[A]
                        Jp = hstack((J, True))
                        added_events = event_index_offset[Jp] + 1
                    queue_index = (scheduled_events_index + int_delay) % _max_delay
                    event_index = event_index_offset + num_scheduled_events[queue_index]
                    if sparse:
                        if precomp:
                            target_indices = self.target_indices[i]
                        else:
                            target_indices = dvecrow.ind[sort_indices]
                    else:
                        target_indices = sort_indices
                    ptr[queue_index, event_index] = target_indices
                    if precomp_weights:
                        sorted_Wrow = self.sorted_Wrow[i]
                    else:
                        sorted_Wrow = Wrow[sort_indices]
                    val[queue_index, event_index] = sorted_Wrow
                    num_scheduled_events[queue_index[Jp]] += added_events

if 0:
    C = ScheduledEventDelayConnection(G, H, structure='dense', max_delay=1 * ms)
    C.connect_full(G, H, weight=1, delay=(0 * ms, 1 * ms))
    
    G.spikemon = SpikeMonitor(G)
    H.spikemon = SpikeMonitor(H)
    G.statemon = MultiStateMonitor(G, record=True)
    H.statemon = MultiStateMonitor(H, record=True)
    
    G.V = 1
    
    run(1.2 * ms)
    
    print 'H.spikemon.nspikes', H.spikemon.nspikes
    print H.spikemon.spikes
    print asarray(C.delay) / ms
    
    subplot(221)
    raster_plot(G.spikemon)
    subplot(222)
    raster_plot(H.spikemon)
    subplot(223)
    G.statemon.plot()
    subplot(224)
    H.statemon.plot()
    show()
else:
    # speed test
    import time, gc, random, numpy
    do_profile = False
    set_global_preferences(useweave=False, usenewpropagate=False)
    for name, Conn in [
                       ('Normal', DelayConnection),
                       ('Events', ScheduledEventDelayConnection),
                       ]:
        clear(True, True)
        reinit_default_clock()
        gc.collect()
        random.seed(2134234)
        numpy.random.seed(324342)
        eqs = '''
        dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
        dge/dt = -ge/(5*ms) : volt
        dgi/dt = -gi/(10*ms) : volt
        '''
        
        P = NeuronGroup(4000, model=eqs,
                      threshold= -50 * mV, reset= -60 * mV)
        P.v = -60 * mV + 10 * mV * rand(len(P))
        Pe = P.subgroup(3200)
        Pi = P.subgroup(800)
        
        Ce = Conn(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02, delay=(0*ms, 5*ms), structure='sparse')
        Ci = Conn(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02, delay=(0*ms, 5*ms), structure='sparse')

        M = SpikeCounter(P)
        
        run(1*ms)
        
        start = time.time()
        
        try:
            if do_profile:
                import cProfile as profile
                import pstats
                profile.run('run(.1*second)','scheduled_events.prof')
                stats = pstats.Stats('scheduled_events.prof')
                stats.strip_dirs()
                stats.sort_stats('cumulative', 'calls')
                stats.print_stats(50)
                import os
                os.remove('scheduled_events.prof')
            else:
                run(.1*second)
        except IndexError:
            print sum(M.count)
            raise
        
        print name, sum(M.count), time.time()-start



from brian import *

def testexponentialcurrent():
    '''Tests whether an exponential current works as predicted
    
    From Tutorial 2b.
    
    The scheme we implement is the following diffential equations:
    
        | taum dV/dt = -V + ge - gi
        | taue dge/dt = -ge
        | taui dgi/dt = -gi
    
    An excitatory neuron connects to state ge, and an inhibitory neuron connects
    to state gi. When an excitatory spike arrives, ge instantaneously increases,
    then decays exponentially. Consequently, V will initially but continuously
    rise and then fall. Solving these equations, if V(0)=0, ge(0)=g0 corresponding
    to an excitatory spike arriving at time 0, and gi(0)=0 then:
    
        | gi = 0    
        | ge = g0 exp(-t/taue)
        | V = (exp(-t/taum) - exp(-t/taue)) taue g0 / (taum-taue)
    '''
    reinit_default_clock()
    taum = 20 * ms
    taue = 1 * ms
    taui = 10 * ms
    Vt = 10 * mV
    Vr = 0 * mV

    spiketimes = [(0, 0 * ms)]

    G1 = SpikeGeneratorGroup(2, spiketimes)
    G2 = NeuronGroup(N=1, model='''
                   dV/dt = (-V+ge-gi)/taum : volt
                   dge/dt = -ge/taue : volt
                   dgi/dt = -gi/taui : volt
                   ''', threshold=Vt, reset=Vr)
    G2.V = Vr

    C1 = Connection(G1, G2, 'ge')
    C2 = Connection(G1, G2, 'gi')

    C1[0, 0] = 3 * mV
    C2[1, 0] = 3 * mV

    Mv = StateMonitor(G2, 'V', record=True)
    Mge = StateMonitor(G2, 'ge', record=True)
    Mgi = StateMonitor(G2, 'gi', record=True)

    run(100 * ms)

    t = Mv.times
    Vpredicted = (exp(-t / taum) - exp(-t / taue)) * taue * (3 * mV) / (taum - taue)

    Vdiff = abs(Vpredicted - Mv[0])

    assert max(Vdiff) < 0.00001

if __name__ == '__main__':
    testexponentialcurrent()



'''
Slope-threshold relationship with noisy inputs, in the adaptive threshold model
-------------------------------------------------------------------------------
Fig. 5E,F from:
Platkiewicz J and R Brette (2011). Impact of Fast Sodium Channel Inactivation on Spike
Threshold Dynamics and Synaptic Integration. PLoS Comp Biol 7(5):
e1001129. doi:10.1371/journal.pcbi.1001129
'''
from brian import *
from scipy import stats,optimize
from scipy.stats import linregress

rectify=lambda x:clip(x/volt,0,Inf)*volt

N=200 # 200 neurons to get more statistics, only one is shown
duration=1*second
#--Biophysical parameters
ENa=60*mV
EL=-70*mV
vT=-55*mV
Vi=-63*mV
tauh=5*ms
tau=5*ms
ka=5*mV
ki=6*mV
a=ka/ki
tauI=5*ms
mu=15*mV
sigma=6*mV/sqrt(tauI/(tauI+tau))

#--Theoretical prediction for the slope-threshold relationship (approximation: a=1+epsilon)
thresh=lambda slope,a: Vi-slope*tauh*log(1+(Vi-vT/a)/(slope*tauh))
#-----Exact calculation of the slope-threshold relationship
thresh_ex=lambda s: optimize.fsolve(lambda th: a*s*tauh*exp((Vi-th)/(s*tauh))-th*(1-a)-a*(s*tauh+Vi)+vT,thresh(s,a))*volt

eqs="""
dv/dt=(EL-v+mu+sigma*I)/tau : volt
dtheta/dt=(vT+a*rectify(v-Vi)-theta)/tauh : volt
dI/dt=-I/tauI+(2/tauI)**.5*xi : 1 # Ornstein-Uhlenbeck
"""
neurons=NeuronGroup(N,eqs,threshold="v>theta",reset='v=EL',refractory=5*ms)
neurons.v=EL
neurons.theta=vT
neurons.I=0
S=SpikeMonitor(neurons)
M=StateMonitor(neurons,'v',record=True)
Mt=StateMonitor(neurons,'theta',record=0)

run(duration,report='text')

# Linear regression gives depolarization slope before spikes
tx=M.times[(M.times>0) & (M.times<1.5*tauh)]
slope,threshold=[],[]
v=array(M._values)
for (i,t) in S.spikes:
    ind=(M.times<t) & (M.times>t-tauh)
    mx=v[:,i][ind]
    s,_,_,_,_=linregress(tx[:len(mx)],mx)
    slope.append(s)
    threshold.append(mx[-1])

# Figure
M.insert_spikes(S) # displays spikes on the trace
subplot(221)
ind=M.times<500*ms
plot(M.times[ind]/ms,M[0][ind]/mV,'k')
plot(Mt.times[ind]/ms,Mt[0][ind]/mV,'r')
xlabel('Time (ms)')
ylabel('Voltage (mV)')

subplot(222)
plot(slope,array(threshold)/mV,'r.')
sx=linspace(0.5*volt/second,4*volt/second,100)
t=array([thresh_ex(s*volt/second) for s in sx])
plot(sx,t/mV,'k')
xlim(0.5,4)
xlabel('Depolarization slope (ms)')
ylabel('Threshold (mV/ms)')

show()



from brian import *
from brian.utils.circular import *
import brianlib as bl
debugmode = False

def f(s):
    if s.__class__ is SpikeContainer:
        t = 'SpikeContainer\n'
        t += '  S: CircularVector(cursor=' + str(s.S.cursor) + ', X=[' + ' '.join(map(str, s.S.X)) + '])\n'
        t += '  ind: CircularVector(cursor=' + str(s.ind.cursor) + ', X=[' + ' '.join(map(str, s.ind.X)) + ']))'
        return t
    else:
        return 'brianlib.' + repr(s)

sc = SpikeContainer(12, 6)
blsc = bl.SpikeContainer(12, 6)
for i in range(20):
    for s in [sc, blsc]:
        x = array([1, 2, 3, 4]) + 10 * i
        s.push(x)
        #s.push(bl.SpikeList(x))
        print f(s)
        print s.get_spikes(0, 0, 200)
        print s.lastspikes()


if False:
    v1 = CircularVector(6, dtype=int)
    v2 = bl.CircularVector(6)
    for v in [v1, v2]:
        v[0:4] = array([1, 2, 3, 4])
        if debugmode:
            print 'First push OK'
            print v
        print v[0:4], v[4:6]
        v.advance(4)
        if debugmode:
            print 'advance 4'
            print v
        print v[0:4], v[4:6]
        v[0:4] = array([5, 6, 7, 8])
        if debugmode:
            print 'Second push OK'
            print v
        print v[0:4], v[4:6]
        v.advance(4)
        if debugmode:
            print 'advance 4'
            print v
        print v[0:4], v[4:6]



from brian import *

from random import sample
from scipy import random
from brian.connection import ConnectionMatrix


class ConstantSparseConnectionMatrix(ConnectionMatrix):
    def __init__(self, dims, val):
        self.sourcelen, self.targetlen = dims
        self.rows = [ array([], dtype=int) for _ in xrange(self.sourcelen) ]
        self.val = val

    def add_row(self, i, X):
        X[self.rows[i]] += self.val

    def add_scaled_row(self, i, X, factor):
        # modulation may not work? need factor[self.rows[i]] here? is factor a number or an array?
        X[self.rows[i]] += factor * self.val

    def random_matrix(self, i_start, i_end, m, offset, p):
        for i in xrange(i_start, i_end):
            k = random.binomial(m, p, 1)[0]
            r = offset + array(sample(xrange(m), k), dtype=int)
            self.rows[i] = r


class ConstantConnection(Connection):
    def __init__(self, source, target, state=0, delay=0 * msecond, modulation=None,
                 weight=1.):
        Connection.__init__(self, source, target, state=state, delay=delay, modulation=modulation)
        self.W = ConstantSparseConnectionMatrix((len(source), len(target)), weight)
        self.weight = weight

    def connect_random(self, P, Q, p):
        weight = self.weight
        try:
            weight + Q._S0[self.nstate]
        except DimensionMismatchError, inst:
            raise DimensionMismatchError("Incorrects unit for the synaptic weights.", *inst._dims)
        i_start = P._origin - self.source._origin
        i_end = i_start + len(P)
        offset = Q._origin - self.target._origin
        m = len(Q)
        self.W.random_matrix(i_start, i_end, m, offset, p)


eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

N = 64000
duration = 500 * ms
usecc = True

Ne = int(0.8 * N)
Ni = N - Ne

P = NeuronGroup(N, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(Ne)
Pi = P.subgroup(Ni)

if usecc:
    Ce = ConstantConnection(Pe, P, 'ge', weight=1.62 * mV)
    Ci = ConstantConnection(Pi, P, 'gi', weight= -9 * mV)
    Ce.connect_random(Pe, P, 0.02)
    Ci.connect_random(Pi, P, 0.02)
else:
    Ce = Connection(Pe, P, 'ge')
    Ci = Connection(Pi, P, 'gi')
    Ce.connect_random(Pe, P, 0.02, weight=1.62 * mV)
    Ci.connect_random(Pi, P, 0.02, weight= -9 * mV)

#M=SpikeMonitor(P)
M = PopulationSpikeCounter(P)

import time
t = time.time()
run(duration)
print time.time() - t
#raster_plot(M)
#show()
print M.nspikes



'''
Sample code showing running multiple processes including multiple GPUs

See Brian/examples/multiprocessing/* for detailed examples with comments.

Notes for Cyrille:

This file works whether or not a GPU is present. If there is a GPU then it
will launch num_gpus processes, otherwise it will launch num_cpus processes.
Things to know:

* Launch one process for each CPU/GPU
* Have a function f and a set of arguments x
* Each value f(x) is computed in a separate process
* The arguments x need to be picklable. Most Brian objects are not, so you'll
  need to pass parameters etc. - enough parameters that you can reconstruct
  the ModelFitting objects in each separate process
* If using the GPU, you need to pass a copy of the process number to each
  process so that it knows which GPU to use
'''

import multiprocessing
try:
    import pycuda.autoinit as autoinit
    import pycuda.driver as drv
    def set_gpu_device(n):
        '''
        This function makes pycuda use GPU number n in the system.
        '''
        global _gpu_context # we make this a global variable so that a
                            # reference is kept to it and the context is not
                            # deallocated
        autoinit.context.pop() # delete the old context before making a new one
        _gpu_context = drv.Device(n).make_context()
    use_gpu = True
except ImportError:
    use_gpu = False

# Note that func actually takes only one argument, that argument is a tuple
# of arguments (which is why there are double parentheses). The reason is that
# pool.map only works for functions of one argument. Note also that the arguments
# have to be picklable and not all Brian objects are, however all numpy objects
# should be.
def func((process_n, x)):
    # if we are using the GPU then there should be one process for each
    # GPU, and process_n should use GPU number process_n.
    if use_gpu:
        set_gpu_device(process_n)
    # just a trivial computation to give the idea
    return x ** 2

if __name__ == '__main__':
    # If we have GPUs we use them, otherwise we use CPUs
    if use_gpu:
        numprocesses = drv.Device.count() # number of GPUs present in the system
        print 'Using %d GPUs' % numprocesses
    else:
        numprocesses = multiprocessing.cpu_count() # number of CPUs in the system
        print 'Using %d CPUs' % numprocesses
    pool = multiprocessing.Pool(processes=numprocesses)
    # args consists of pairs (process_n, x) where process_n is the number of the
    # process, which is used to initialiase the correct GPU if one is present,
    # and x is the argument to the function (see notes in module docstring
    # about this)
    args = zip(range(numprocesses), range(2, numprocesses + 2))
    results = pool.map(func, args) # launches multiple processes
    for (pid, arg), res in zip(args, results):
        print 'Process', pid, 'argument', arg, 'result', res



'''
Sound localization with HRTFs
-----------------------------
Goodman DF and R Brette (2010). Spike-timing-based computation in sound
localization. PLoS Comp Biol 6(11): e1000993. doi:10.1371/journal.pcbi.1000993.

Simplified version of the "ideal" sound localisation model.

The sound is played at a particular spatial location (indicated on the final
plot by a red +). Each location has a corresponding assembly of neurons, whose
summed firing rates give the sizes of the blue circles in the plot. The most
strongly responding assembly is indicated by the green x, which is the estimate
of the location by the model.
'''
from brian import *
from brian.hears import *

# Download the IRCAM database
# http://recherche.ircam.fr/equipes/salles/listen/download.html
# and replace this filename with the location you downloaded it to
hrtfdb = IRCAM_LISTEN(r'Z:\HRTF\IRCAM')
subject = 1002
hrtfset = hrtfdb.load_subject(subject)
# This gives the number of spatial locations in the set of HRTFs
num_indices = hrtfset.num_indices
# Choose a random location for the sound to come from
index = randint(num_indices)
# A sound to test the model with
sound = Sound.whitenoise(500*ms)
# This is the specific HRTF for the chosen location
hrtf = hrtfset.hrtf[index]
# We apply the chosen HRTF to the sound, the output has 2 channels
hrtf_fb = hrtf.filterbank(sound)
# We swap these channels (equivalent to swapping the channels in the
# subsequent filters, but simpler to do it with the inputs)
swapped_channels = RestructureFilterbank(hrtf_fb, indexmapping=[1, 0])
# Now we apply all of the possible pairs of HRTFs in the set to these
# swapped channels, which means repeating them num_indices times first
hrtfset_fb = hrtfset.filterbank(Repeat(swapped_channels, num_indices))
# Now we apply cochlear filtering (logically, this comes before the HRTF
# filtering, but since convolution is commutative it is more efficient to
# do the cochlear filtering afterwards
cfmin, cfmax, cfN = 150*Hz, 5*kHz, 40
cf = erbspace(cfmin, cfmax, cfN)
# We repeat each of the HRTFSet filterbank channels cfN times, so that
# for each location we will apply each possible cochlear frequency
gfb = Gammatone(Repeat(hrtfset_fb, cfN),
                tile(cf, hrtfset_fb.nchannels))
# Half wave rectification and compression
cochlea = FunctionFilterbank(gfb, lambda x:15*clip(x, 0, Inf)**(1.0/3.0))
# Leaky integrate and fire neuron model
eqs = '''
dV/dt = (I-V)/(1*ms)+0.1*xi/(0.5*ms)**.5 : 1
I : 1
'''
G = FilterbankGroup(cochlea, 'I', eqs, reset=0, threshold=1, refractory=5*ms)
# The coincidence detector (cd) neurons
cd = NeuronGroup(num_indices*cfN, eqs, reset=0, threshold=1, clock=G.clock)
# Each CD neuron receives precisely two inputs, one from the left ear and
# one from the right, for each location and each cochlear frequency
C = Connection(G, cd, 'V')
for i in xrange(num_indices*cfN):
    C[i, i] = 0.5                 # from right ear
    C[i+num_indices*cfN, i] = 0.5 # from left ear
# We want to just count the number of CD spikes
counter = SpikeCounter(cd)
# Run the simulation, giving a report on how long it will take as we run
run(sound.duration, report='stderr')
# We take the array of counts, and reshape them into a 2D array which we sum
# across frequencies to get the spike count of each location-specific assembly
count = counter.count
count.shape = (num_indices, cfN)
count = sum(count, axis=1)
count = array(count, dtype=float)/amax(count)
# Our guess of the location is the index of the strongest firing assembly
index_guess = argmax(count)
# Now we plot the output, using the coordinates of the HRTFSet
coords = hrtfset.coordinates
azim, elev = coords['azim'], coords['elev'] 
scatter(azim, elev, 100*count)
plot([azim[index]], [elev[index]], '+r', ms=15, mew=2)
plot([azim[index_guess]], [elev[index_guess]], 'xg', ms=15, mew=2)
xlabel('Azimuth (deg)')
ylabel('Elevation (deg)')
xlim(-5, 350)
ylim(-50, 95)
show()



'''
Information theory measures.

Adapted by R Brette from several papers (see in the code).
Uses the ANN wrapper in scikits.
'''
import scikits.ann as ann
from scipy.special import gamma, psi
from scipy.linalg import det, inv
from scipy import *

__all__ = ['nearest_distances', 'entropy', 'mutual_information', 'entropy_gaussian',
         'mutual_information2']

def nearest_distances(X, k=1):
    '''
    X = array(N,M)
    N = number of points
    M = number of dimensions
    
    returns the squared distance to the kth nearest neighbor for every point in X
    '''
    ktree = ann.kdtree(X)
    _, d = ktree.knn(X, k + 1) # the first nearest neighbor is itself
    return d[:, -1] # returns the distance to the kth nearest neighbor

def entropy_gaussian(C):
    '''
    Entropy of a gaussian variable with covariance matrix C
    '''
    if isscalar(C): # C is the variance
        return .5 * (1 + log(2 * pi)) + .5 * log(C)
    else:
        n = C.shape[0] # dimension
        return .5 * n * (1 + log(2 * pi)) + .5 * log(abs(det(C)))

def entropy(X, k=1):
    '''
    Returns the entropy of X,
    given as array X = array(n,dx)
    where
      n = number of samples
      dx = number of dimensions
    
    Optionally:
      k = number of nearest neighbors for density estimation
    '''
    # Distance to kth nearest neighbor
    r = nearest_distances(X, k) # squared distances
    n, d = X.shape
    volume_unit_ball = (pi ** (.5 * d)) / gamma(.5 * d + 1)
    '''
    F. Perez-Cruz, (2008). Estimation of Information Theoretic Measures
    for Continuous Random Variables. Advances in Neural Information
    Processing Systems 21 (NIPS). Vancouver (Canada), December.
     
    return .5*d*mean(log(r))+log(volume_unit_ball)+log(n-1)-log(k)
    '''
    '''
    Kozachenko, L. F. & Leonenko, N. N. 1987 Sample estimate of entropy
    of a random vector. Probl. Inf. Transm. 23, 95-101.
    See also: Evans, D. 2008 A computationally efficient estimator for
    mutual information, Proc. R. Soc. A 464 (2093), 1203-1215.
    and:
    Kraskov A, Stogbauer H, Grassberger P. (2004). Estimating mutual
    information. Phys Rev E 69(6 Pt 2):066138.
    '''
    return .5 * d * mean(log(r)) + log(volume_unit_ball) + psi(n) - psi(k)

def mutual_information(X, Y, k=1):
    '''
    Returns the mutual information between X and Y.
    Each variable is a matrix X = array(n,dx)
    where
      n = number of samples
      dx,dy = number of dimensions
    
    Optionally, the following keyword argument can be specified:
      k = number of nearest neighbors for density estimation
    '''
    return entropy(X) + entropy(Y) - entropy(hstack((X, Y)), k=k)

def mutual_information2(X, p):
    '''
    Mutual information of continuous variable X and discrete
    variable Y taking p values with equal probabilities.
    X is a matrix (n,d) such that X[(n/p)*i:(n/p)*(i+1),:] are the samples
    for the same value of Y.
    
    Maybe to a principal component analysis?
    (problem of highly correlated variables)
    
    Method:
    * Empirical average I=<log(p(X|Y)/p(X))>
    * p(X) and p(X|Y) are estimated with Gaussian assumptions
    '''
    X = X / X.max()
    n, d = X.shape
    M = mean(X, axis=0).reshape((d, 1))
    C = cov(X.T)
    C += C.max()*.001 * eye(d) # avoids degeneracy (hack!)
    invC = inv(C)
    m = int(n / p) # number of samples / value of Y
    I = 0.
    for i in range(p):
        Z = X[m * i:m * (i + 1), :] # same Y
        MZ = mean(Z, axis=0).reshape((d, 1))
        CZ = cov(Z.T)
        CZ += CZ.max()*.001 * eye(d) # avoids degeneracy (hack!)
        #try:
        invCZ = inv(CZ)
        #except LinAlgError: # singular matrix!
        #    
        #A=invC-invCZ
        Ii = 0.
        for j in range(m):
            x = X[m * i + j, :].reshape((d, 1))
            Ii = dot(dot(x.T - M.T, invC), x - M) - dot(dot(x.T - MZ.T, invCZ), x - MZ)
            #Ii+=dot(dot(x.T,A),x)
        Ii = .5 * (log(det(C * invCZ)) + 1) + Ii / m
        I += Ii
    I = I / p
    return I[0, 0]

if __name__ == '__main__':
    '''
    Testing against correlated Gaussian variables
    (analytical results are known)
    '''
    from scipy import *
    # Entropy of a 15-dimensional gaussian variable
    n = 10000
    d = 15
    P = randn(d, d) # change of variables
    C = dot(P, P.T)
    Y = randn(d, n)
    X = dot(P, Y)
    H = entropy_gaussian(C)
    Hest = entropy(X.T, k=5)
    print Hest, H

    # Mutual information between two correlated gaussian variables
    P = randn(2, 2)
    C = dot(P, P.T)
    U = randn(2, n)
    Z = dot(P, U).T
    X = Z[:, 0]
    X = X.reshape(len(X), 1)
    Y = Z[:, 1]
    Y = Y.reshape(len(Y), 1)
    # in bits
    print mutual_information(X, Y, k=5) / log(2.), \
          (entropy_gaussian(C[0, 0]) + entropy_gaussian(C[1, 1]) - entropy_gaussian(C)) / log(2.)



import numpy, scipy
from numpy import *
from brian import *

#physical characteristics
Cap = 2 * uF / (cm * cm)
GNa = 0.02 * siemens / (cm * cm)
GK = 0.02 * siemens / (cm * cm)
GLeak = 0.002 * siemens / (cm * cm)
ELeak = -70 * mV
EK = -100 * mV
ENa = 50 * mV
#ML characteristics
v1 = -11.2 * mV
v2 = 18 * mV
v3 = -20 * mV
v4 = 10 * mV
phiScalingFactor = 0.15 * mV / mV
TauCofactor = 2 * ms

eqs = Equations('''
dvm/dt=(-GNa*minf*(vm-ENa)-GK*w*(vm-EK)-GLeak(vm-ELeak))/200*pF:mV
minf=0.5*(1+tanh((vm-v1)/v2)):1
winf=0.5*(1+tanh((vm-v3)/v4)):1
TauW=TauCofactor/cosh((vm-v3)/(2*v4)):ms
dw/dt=phiScalingFactor*(winf-w)/TauW:kHz''')
#eqs.compile_functions(True)
neuron = NeuronGroup(1, eqs, freeze=True, method='Euler')



#!/usr/bin/env python
#import brian_no_units
from brian import *
from brian.library.IF import *
from brian.experimental.general_stdp import *
from brian.experimental.heterog_delay import *
from brian.experimental.stdp_sparse import *
from brian.connection import DenseConnectionMatrix
import numpy
import time

#set_global_preferences(useweave=True)

defaultclock.dt = 0.5 * ms

class VariableIzhikevichReset(object):
    def __call__(self, P):
        spikes = P.LS.lastspikes()
        P.vm_[spikes] = Vr
        P.w_[spikes] += P.d_[spikes]

T = 10000 * ms

#Network parameters
N = 1000 # total number of neurons
Ne = 800 # number of excitatory neurons
Ni = 200 # number of inhibitory neurons
Nc = 100 # number of synapses per neuron

vm0 = -65 * mV # the initial membrane current
w0 = 0.2 * vm0 / ms # the initial recovery varible
Vr = -65 * mV
Vt = 30 * mV # "firing" threshold
min_delay = 1 * ms
max_delay = 20 * ms # maximum axonal delay
i_delay = 1 * ms # the inhibitory delay
wE0 = 6 * mV #initial positive synaptic weight
wI0 = -5 * mV #initial negative synaptic weight
# I think these are wrong, the units should be mV
#wE0= 6*nA #initial positive synaptic weight
#wI0= -5*nA #initial negative synaptic weight

#STDP
gmax = 10 * mV#0.010 # maximum synaptic weight
tau_pre = 20 * ms
tau_post = 20 * ms
dA_pre = gmax * .005
dA_post = -dA_pre * 1.05

eqs = Izhikevich(a='a', b=0.2 / ms) + '''
        a : Hz
        d : volt/second
        '''
model = Model(model=eqs, threshold=Vt, reset=VariableIzhikevichReset())

#dt bit needed for versions of Brian which aren't recent
G = NeuronGroup(N, model, max_delay=i_delay + defaultclock.dt, compile=True, freeze=True)

Ge = G.subgroup(Ne)
Gi = G.subgroup(Ni)

Ge.vm = vm0
Ge.w = w0
Ge.a = 0.02 / ms
Ge.d = 8.0 * mV / ms

Gi.vm = vm0
Gi.w = w0
Gi.a = 0.1 / ms
Gi.d = 2.0 * mV / ms

start_time = time.time()
Ce = HeterogeneousDelayConnection(Ge, G, max_delay=max_delay, delays_per_synapse=True)
#Cei = HeterogeneousDelayConnection(Ge, Gi, max_delay=max_delay, delays_per_synapse=True)
Cie = Connection(Gi, Ge, delay=1 * ms)

# Add Nc connections to each neuron in the network
#Ce.connect_random(Ge, Ge, p=float(Nc)/N, weight=wE0)
#Ce.connect_random(Ge, Gi, p=float(Nc)/N, weight=wI0)#really want this to be wI0?
Ce.connect_random(Ge, G, p=float(Nc) / N, weight=wE0)
for i in range(len(Ge)):
    Ce[i, i] = 0
Ce.delayvec = DenseConnectionMatrix((len(Ge), len(G))) # faster in the current implementaton I think

# This way is slow, but if we're making delayvec a sparse matrix we have to do it like this
#for i in xrange(len(Ge)):
#    for j in xrange(len(G)):
#        if abs(Ce[i,j])>1e-30:
#            Ce.delayvec[i,j]=rand()*(max_delay-min_delay)+min_delay

Ce.delayvec[:] = random((len(Ge), len(G))) * (max_delay - min_delay) + min_delay
# Use this if you need to be sure they are integer multiples of 1ms, but not necessary I think
#Ce.delayvec[:] = ((random((len(Ge), len(G)))*max_delay/ms).astype(int)).astype(float)*ms

# An alternative idea: SparseSTDPConnectionMatrix faster for delayvec too? Answer: not noticeably
#Ce.delayvec = SparseSTDPConnectionMatrix(Ce.W)
#Ce.delayvec[:] = rand(len(Ce.delayvec[:]))*(max_delay-min_delay)+min_delay
#Ce.delayvec.ndim = 2

Cie.connect_random(Gi, Ge, p=float(Nc) / N, weight=wI0)

# if you turn STDP off it takes 20 times longer! The SparseSTDPConnectionMatrix is MORE efficient than
# what was there before!!
STDPe = SongAbbottSTDP(Ce, gmax=gmax, tau_pre=tau_pre, tau_post=tau_post,
                      dA_pre=dA_pre, dA_post=dA_post)
Ce.W = SparseSTDPConnectionMatrix(Ce.W)

print "Network build time:", time.time() - start_time, "seconds"

thalamic_alternate = True

Gvm_ = G.vm_
thaldelta = float(20 * mV)

@network_operation(when='start')
def thalamic_input():
    global thalamic_alternate
    # at each cycle, some random neuron gets a 20*mV boost
    if thalamic_alternate:
        n = int(N * random())
        Gvm_[n] += thaldelta#float(20*mV)
        thalamic_alternate = False
    else:
        thalamic_alternate = True

S = SpikeMonitor(G)

net = MagicNetwork()
#net.update_schedule_groups_resets_connections()

def do_run(T=T, doplot=True):
    start_time = time.time()
    net.run(T)
    print "Excecution time:", time.time() - start_time, "seconds"
    if doplot:
        raster_plot(S)
        show()

do_run()

#import cProfile
#cProfile.run('do_run(1000*ms,False)')

#import pycallgraph
#cg_func = 'Network.update'
#def ff(pat):
#    def f(call_stack, module_name, class_name, func_name, full_name):
#        if not 'brian' in module_name: return False
#        for n in call_stack+[full_name]:
#            if pat in n:
#                return True
#        return False
#    return f
#pycallgraph.start_trace(filter_func=ff(cg_func))
#do_run(1*second,False)
#pycallgraph.stop_trace()
#pycallgraph.make_dot_graph('callgraph-'+cg_func+'.png')



from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
                    reshape, sum, log, Monitor, NetworkOperation, defaultclock, linspace, vstack, \
                    arange, sort_spikes, rint, SpikeMonitor, Connection, int32, double,SpikeGeneratorGroup,DelayConnection,\
                    SpikeCounter,forget,diagflat,inf,sqrt,ones_like,isnan,append,repeat,tile,mean,logical_and,where
from brian.tools.statistics import firing_rate, get_gamma_factor
from playdoh import *
try:
    import pycuda
    import pycuda.driver as drv
    import pycuda
    from pycuda import gpuarray
    can_use_gpu = True
except ImportError:
    can_use_gpu = False
from brian.experimental.codegen.integration_schemes import *
import sys, cPickle




class Criterion(Monitor, NetworkOperation):
    """
    Abstract class from which modelfitting criterions should derive.
    Derived classes should implement the following methods:
    
    ``initialize(self, **params)``
        Called once before the simulation. ```params`` is a dictionary with criterion-specific
        parameters. 
    
    ``timestep_call(self)``
        Called at every timestep.
    
    ``spike_call(self, neurons)``
        Called at every spike, with the spiking neurons as argument.
    
    ``get_values(self)``
        Called at the end, returns the criterion values.
        
    You have access to the following methods:
    
    ``self.get_value(self, varname)``
        Returns the value of the specified variable for all neurons (vector).
    
    You have access to the following attributes:
    
    ``self.step``
        The time step (integer)
    
    ``self.group``
        The NeuronGroup
    
    ``self.traces=None``
        Target traces.
        A 2-dimensional K*T array where K is the number of targets, and T the total number of timesteps.
        It is still a 2-dimensional array when K=1. It is None if not specified.
        
    ``self.spikes=None``
        A list of target spike trains : [(i,t)..] where i is the target index and t the spike time.
    
    ``self.N``
        The number of neurons in the NeuronGroup
    
    ``self.K=1``
        The number of targets.
    
    ``self.duration``
        The total duration of the simulation, in seconds.
    
    ``self.total_steps``
        The total number of time steps.
    
    ``self.dt``
        The timestep duration, in seconds.
    
    ``self.delays=zeros(self.n)``
        The delays for every neuron, in seconds. The delay is relative to the target.
    
    ``self.onset=0*ms``
        The onset, in seconds. The first timesteps, before onset, should be discarded in the criterion.
    
    ``self.intdelays=0``
        The delays, but in number of timesteps (``int(delays/dt)``).
    
    NOTE: traces and spikes have all been sliced before being passed to the criterion object
    """
    def __init__(self, group, traces=None, spikes=None, targets_count=1, duration=None, onset=0*ms, 
                 spikes_inline=None, spikes_offset=None,trials_offset=None,
                 traces_inline=None, traces_offset=None,
                 delays=None, when='end', **params):
        NetworkOperation.__init__(self, None, clock=group.clock, when=when)
        self.group = group
        # needed by SpikeMonitor
        self.source = group
        self.source.set_max_delay(0)
        self.delay = 0
        self.trials_offset=trials_offset
        self.N = len(group) # number of neurons
        self.K = targets_count # number of targets
        self.dt = self.clock.dt
        if traces is not None:
            self.traces = array(traces) # KxT array
            if self.traces.ndim == 1:
                self.traces = self.traces.reshape((1,-1))
            assert targets_count==self.traces.shape[0]
        self.spikes = spikes
        # get the duration from the traces if duration is not specified in the constructor
        if duration is None: duration = self.traces.shape[1] # total number of steps
        self.duration = duration
        self.total_steps = int(duration/self.dt)
        if delays is None: delays = zeros(self.n)
        self.delays = delays
        self.onset = int(onset/self.dt)
        self.intdelays = array(self.delays/self.clock.dt, dtype=int)
        self.mindelay = min(delays)
        self.maxdelay = max(delays)
        # the following data is sliced
        self.spikes_inline = spikes_inline
        self.spikes_offset = spikes_offset
        self.traces_inline = traces_inline
        self.traces_offset = traces_offset
        if self.spikes is not None:
            # target spike count and rates
            self.target_spikes_count = self.get_spikes_count(self.spikes)
            self.target_spikes_rate = self.get_spikes_rate(self.spikes)
        self.initialize(**params)

    def step(self):
        """
        Return the current time step
        """
        return int(round(self.clock.t/self.dt))
    
    def get_spikes_count(self, spikes):
        
        count = zeros(self.K)
        for (i,t) in spikes:
            count[i] += 1
        return count
    
    def get_spikes_rate(self, spikes):
        count = self.get_spikes_count(spikes)
        return count*1.0/self.duration
    
    def get_gpu_code(self): # TO IMPLEMENT
        """
        Returns CUDA code snippets to put at various places in the CUDA code template.
        It must return a dictionary with the following items:
            %CRITERION_DECLARATION%: kernel declaration code
            %CRITERION_INIT%: initialization code
            %CRITERION_TIMESTEP%: main code, called at every time step
            %CRITERION_END%: finalization code
        """
        log_warn("GPU code not implemented by the derived criterion class")
    
    def initialize_cuda_variables(self): # TO IMPLEMENT
        """
        Initialize criterion-specific CUDA variables here.
        """
        pass
    
    def get_kernel_arguments(self): # TO IMPLEMENT
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        pass
    
    def initialize(self): # TO IMPLEMENT
        """
        Override this method to initialize the criterion before the simulation
        """
        pass
    
    def timestep_call(self): # TO IMPLEMENT
        """
        Override this method to do something at every time step
        """
        pass
    
    def spike_call(self, neurons): # TO IMPLEMENT
        """
        Override this method to do something at every time spike. neurons contains
        the list of neurons that just spiked
        """
        pass
    
    def get_value(self, varname):
        return self.group.state_(varname)
    
    def __call__(self):
        self.timestep_call()
        neurons = self.group.get_spikes()
        self.spike_call(neurons)
        
#    def propagate(self, neurons):
#        self.spike_call(neurons)

    def get_values(self): # TO IMPLEMENT
        """
        Override this method to return the criterion values at the end.
        It must return one (or several, as a tuple) additive values, i.e.,
        values corresponding to slices. The method normalize can normalize
        combined values.
        """
        pass
    additive_values = property(get_values)

    def normalize(self, values): # TO IMPLEMENT
        """
        Values contains combined criterion values. It is either a vector of values (as many values
        as neurons), or a tuple with different vector of as many values as neurons.
        """
        pass

class CriterionStruct(object):
    type = None # 'trace', 'spike' or 'both'
    def get_name(self):
        return self.__class__.__name__
    name = property(get_name)




class LpErrorCriterion(Criterion):
    type = 'traces'
    def initialize(self, p=2, varname='v', method='all',insets=None,outsets=None,points=None):
        """
        Called at the beginning of every iteration. The keyword arguments here are
        specified in modelfitting.initialize_criterion().
        """
        self.method=method
        if insets is not None:
            self.insets=array(append(insets,insets[-1]+100000000),dtype=int)
            self.outsets=array(append(outsets,outsets[-1]+100000000),dtype=int)
        if points is not None:
            self.points=array(append(points,points[-1]+100000000),dtype=int)
        self.next_index=0
        self.p = double(p)
        self.varname = varname

        self._error = zeros((self.K, self.N))
    
    def timestep_call(self):
        if self.method=='all':
            v = self.get_value(self.varname)
            t = self.step()+1
            if t<self.onset: return # onset
            if t*self.dt >= self.duration: return
            d = self.intdelays
            indices = (t-d>=0)&(t-d<self.total_steps) # neurons with valid delays (stay inside the target trace)
            vtar = self.traces[:,t-d] # target value at this timestep for every neuron
            for i in xrange(self.K):
                self._error[i,indices] += abs(v[indices]-vtar[i,indices])**self.p
                
        if self.method=='intervals':
            v = self.get_value(self.varname)
            t = self.step()+1
            if t<self.onset: return # onset
            if t*self.dt >= self.duration: return
            if t>=self.insets[self.next_index] and t<=self.outsets[self.next_index]:
                if t==self.outsets[self.next_index]:
                    self.next_index+=1
                d = self.intdelays
                indices = (t-d>=0)&(t-d<self.total_steps) # neurons with valid delays (stay inside the target trace)
                vtar = self.traces[:,t-d] # target value at this timestep for every neuron
                for i in xrange(self.K):
                    self._error[i,indices] += abs(v[indices]-vtar[i,indices])**self.p
                    
        if self.method=='points':
            v = self.get_value(self.varname)
            t = self.step()+1
            if t<self.onset: return # onset
            if t*self.dt >= self.duration: return
            if t==self.points[self.next_index]:
                
                self.next_index+=1
                d = self.intdelays
                indices = (t-d>=0)&(t-d<self.total_steps) # neurons with valid delays (stay inside the target trace)
                vtar = self.traces[:,t-d] # target value at this timestep for every neuron
                for i in xrange(self.K):
                    self._error[i,indices] += abs(v[indices]-vtar[i,indices])**self.p
                    
    def get_cuda_code(self):
        code = {}
        if self.method=='all':
            #  DECLARATION
            code['%CRITERION_DECLARE%'] = """
            double *error_arr,"""
            
            # INITIALIZATION
            code['%CRITERION_INIT%'] = """
            double error = error_arr[neuron_index];"""
            
            # TIMESTEP
            code['%CRITERION_TIMESTEP%'] = """
            if ((T >= onset)&(Tdelay<duration-spikedelay-1)) {
                error = error + pow(abs(trace_value - %s), %.4f);
            }
            """ % (self.varname, self.p)
            
            # FINALIZATION
            code['%CRITERION_END%'] = """
            error_arr[neuron_index] = error;"""
        
        if self.method=='intervals':
            #  DECLARATION
            code['%CRITERION_DECLARE%'] = """
            double *error_arr,
            int *insets,
            int *outsets,
            int *next_index_past,"""
            
            # INITIALIZATION
            code['%CRITERION_INIT%'] = """
            double error = error_arr[neuron_index];
            int next_index=*next_index_past;
            int inset=insets[next_index];
            int outset=outsets[next_index]; """
            
             #TIMESTEP
            code['%CRITERION_TIMESTEP%'] = """
           if ((T >= onset)&(Tdelay<duration-spikedelay-1)&(T >= inset)&(T <= outset)) {
                if (T == outset)
                    {next_index+=1;
                    inset=insets[next_index];
                    outset=outsets[next_index];
                    }
                 error = error + pow(abs(trace_value - %s), %.4f);
                 }
            
            """ %(self.varname, self.p)
      
            
            # FINALIZATION
            code['%CRITERION_END%'] = """
            error_arr[neuron_index] = error;
            *next_index_past=next_index;
            """
            
        if self.method=='points':
            #  DECLARATION
            code['%CRITERION_DECLARE%'] = """
            double *error_arr,
            int *points,
            int *next_index_past,"""
            
            # INITIALIZATION
            code['%CRITERION_INIT%'] = """
            double error = error_arr[neuron_index];
            int next_index=*next_index_past;
            int point=points[next_index]; """
            
            # TIMESTEP
            #(T >= onset)&(Tdelay<duration-spikedelay-1)&
            code['%CRITERION_TIMESTEP%'] = """
            if ((T==point)) {
                next_index+=1;
                point=points[next_index];
                error = error + pow(abs(trace_value - %s), %.4f);
            }
            """% (self.varname, self.p)
            
            
            # FINALIZATION
            code['%CRITERION_END%'] = """
            error_arr[neuron_index] = error;
            *next_index_past = next_index;
            """

        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.error_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
        if self.method=='intervals':
            self.insets_gpu = gpuarray.to_gpu(array(self.insets, dtype=int32))
            self.outsets_gpu = gpuarray.to_gpu(array(self.outsets, dtype=int32))
            self.next_index_past = gpuarray.to_gpu(array(int32(0)))
        if self.method=='points':
            self.points_gpu = gpuarray.to_gpu(array(self.points, dtype=int32))
            self.next_index_past = gpuarray.to_gpu(array(int32(0)))
            
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.error_gpu]
        if self.method=='intervals':
            args += [self.insets_gpu,self.outsets_gpu,self.next_index_past]
        if self.method=='points':
            args += [self.points_gpu,self.next_index_past]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self._error = self.error_gpu.get()
        if self.method=='intervals' or self.method=='points':
            self.index=self.next_index_past.get()
    
    def get_values(self):
        if self.K == 1: error = self._error.flatten()
        else: error = self._error
        #print self.index
        return error # just the integral, for every slice
    
    def normalize(self, error):
        # error is now the combined error on the whole duration (sum on the slices)
        # HACK: 1- because modelfitting MAXIMIZES for now...
#        self._norm = sum(self.traces**self.p, axis=1)**(1./self.p) # norm of every trace
        return 1-(self.dt*error)**(1./self.p)#/self._norm

class LpError(CriterionStruct):
    """
    Structure used by the users to specify a criterion
    """
    def __init__(self, p = 2, varname = 'v', method='all',insets=None,outsets=None,points=None):
        self.type = 'trace'
        self.p = p
        self.varname = varname
        self.method = method
        self.insets = insets
        self.outsets = outsets
        self.points=points


class GammaFactorCriterion2(Criterion):

    type = 'spikes'
    def initialize(self, delta=4*ms, fr_weight=0,coincidence_count_algorithm='exclusive',nlevels=1,level_duration=100):
        self.algorithm = coincidence_count_algorithm
        self.delta = int(rint(delta / self.dt))

        self.fr_weight = fr_weight
        self.spike_count = zeros(self.N, dtype='int')
        self.coincidences = zeros(self.N, dtype='int')
        self.spiketime_index = self.spikes_offset

        self.last_spike_time = array(rint(self.spikes_inline[self.spiketime_index] / self.dt), dtype=int)
        self.next_spike_time = array(rint(self.spikes_inline[self.spiketime_index + 1] / self.dt), dtype=int)
        self.ntrials = len(self.trials_offset)
        self.nlevels = nlevels
        self.level_duration=rint(level_duration/ self.dt)
        
        self.target_count=zeros(self.ntrials)
        self.target_rate=zeros(self.ntrials)
        ind_start=nonzero(self.spikes_inline==-1)[0]
        ind_start=append(ind_start,len(self.spikes_inline))
        self.trials_offset=tile(self.trials_offset,self.N)
        for itrial in xrange(self.ntrials):
            self.target_count[itrial]=ind_start[itrial+1]-ind_start[itrial]-2
            self.target_rate[itrial]=self.target_count[itrial]/self.spikes_inline[ind_start[itrial+1]-2]
        
        self.target_count_level=zeros((self.nlevels,self.ntrials))
        for ilevel in xrange(self.nlevels):
            for itrial in xrange(self.ntrials):
                temp=rint(self.spikes_inline[ind_start[itrial]+1:ind_start[itrial+1]-1]/self.dt)
#                print ilevel*self.level_duration,(ilevel+1)*self.level_duration,max(temp)
                ind=logical_and(temp>ilevel*self.level_duration,temp<=(ilevel+1)*self.level_duration)
#                print where(ind)[0]
                self.target_count_level[ilevel,itrial]=len(where(ind)[0])
        # First target spikes (needed for the computation of 
        #   the target train firing rates)
#        self.first_target_spike = zeros(self.N)

        self.last_spike_allowed = ones(self.N, dtype='bool')
        self.next_spike_allowed = ones(self.N, dtype='bool')
        
    
    if 1:
        #threadIdx.x+blockDim.x*itrial
        def get_cuda_code(self):
            block_size=32
            code = {}
            
            #  DECLARATION
            code['%CRITERION_DECLARE%'] = """
        int *spikecount,          // Number of spikes produced by each neuron
        int *num_coincidences,    // Count of coincidences for each neuron
            """
            
            if self.algorithm == 'exclusive':
                code['%CRITERION_DECLARE%'] += """ 
        int *sp_trial_indices,   // 
        int *last_spike_time_arr,  // pass zero array
        int *next_spike_time_arr,
        bool *last_spike_allowed_arr,
        bool *next_spike_allowed_arr,
        int *spike_count_level,
        int *current_level_index,
               """
            
            # INITIALIZATION
            code['%CRITERION_INIT%'] = """
         int ntrials=%i;//*ntrial;
         int nlevel=%i;
         int level_duration=%i;
        int level_ind = current_level_index[neuron_index];  
        int nspikes_level = spike_count_level[neuron_index*nlevel+level_ind];
        
        int nspikes = spikecount[neuron_index];   
        __shared__ int ncoinc[%i];
        __shared__ int sp_trial_indices_temp[%i];
        __shared__ int last_spike_time[%i];
        __shared__ int next_spike_time[%i];
        __shared__ bool last_spike_allowed[%i];
        __shared__ bool next_spike_allowed[%i];
    
       for(int itrial=0;itrial<ntrials;itrial++)
            {
            sp_trial_indices_temp[threadIdx.x*ntrials+itrial] = sp_trial_indices[neuron_index*ntrials+itrial];
            last_spike_time[threadIdx.x*ntrials+itrial] = last_spike_time_arr[neuron_index*ntrials+itrial];
            next_spike_time[threadIdx.x*ntrials+itrial] =next_spike_time_arr[neuron_index*ntrials+itrial];
            last_spike_allowed[threadIdx.x*ntrials+itrial] = last_spike_allowed_arr[neuron_index*ntrials+itrial];
            next_spike_allowed[threadIdx.x*ntrials+itrial] = next_spike_allowed_arr[neuron_index*ntrials+itrial];
            ncoinc[threadIdx.x*ntrials+itrial] = num_coincidences[neuron_index*ntrials+itrial];
            }  
       __syncthreads();
            """%(self.ntrials,self.nlevels,self.level_duration,self.ntrials*block_size,self.ntrials*block_size,self.ntrials*block_size,self.ntrials*block_size,self.ntrials*block_size,self.ntrials*block_size)
            
            
            # TIMESTEP
            code['%CRITERION_TIMESTEP%'] = """
            const int Tspike = T+spikedelay;
            """
            
            if self.algorithm == 'exclusive':
                code['%CRITERION_TIMESTEP%'] += """
           for(int itrial=0;itrial<ntrials;itrial++)
            {
                bool near_last_spike = last_spike_time[threadIdx.x*ntrials+itrial]+%d>=Tspike;
                bool near_next_spike = next_spike_time[threadIdx.x*ntrials+itrial]-%d<=Tspike;
                near_last_spike = near_last_spike && has_spiked;
                near_next_spike = near_next_spike && has_spiked;
    
                ncoinc[threadIdx.x*ntrials+itrial] +=(near_last_spike&&last_spike_allowed[threadIdx.x*ntrials+itrial]) || (near_next_spike&&next_spike_allowed[threadIdx.x*ntrials+itrial]);
                bool near_both_allowed = (near_last_spike&&last_spike_allowed[threadIdx.x*ntrials+itrial]) && (near_next_spike&&next_spike_allowed[threadIdx.x*ntrials+itrial]);
                last_spike_allowed[threadIdx.x*ntrials+itrial] = last_spike_allowed[threadIdx.x*ntrials+itrial] && !near_last_spike;
                next_spike_allowed[threadIdx.x*ntrials+itrial] = (next_spike_allowed[threadIdx.x*ntrials+itrial] && !near_next_spike) || near_both_allowed;
        
                if(Tspike>=next_spike_time[threadIdx.x*ntrials+itrial]){
                    sp_trial_indices_temp[threadIdx.x*ntrials+itrial]++;
                    last_spike_time[threadIdx.x*ntrials+itrial] = next_spike_time[threadIdx.x*ntrials+itrial];  
                    next_spike_time[threadIdx.x*ntrials+itrial] = spiketimes[sp_trial_indices_temp[threadIdx.x*ntrials+itrial]+1];
                }
                last_spike_allowed[threadIdx.x*ntrials+itrial] = next_spike_allowed[threadIdx.x*ntrials+itrial];
                next_spike_allowed[threadIdx.x*ntrials+itrial] = true;
     
            }

            nspikes += has_spiked*(T>=onset);
           if (T>(level_ind+1)*level_duration)
            {
            spike_count_level[neuron_index*nlevel+level_ind]=nspikes_level;
            level_ind+=1;
            nspikes_level=spike_count_level[neuron_index*nlevel+level_ind];
            }
            nspikes_level += has_spiked*(T>=onset);
           
            """% (self.delta, self.delta)
            
            # FINALIZATION
            code['%CRITERION_END%'] = """
            spike_count_level[neuron_index*nlevel+level_ind]=nspikes_level;
            if (Tend>(level_ind+1)*level_duration)
            {
            level_ind+=1;
            }
            
            for(int itrial=0;itrial<ntrials;itrial++)
                {
                sp_trial_indices[neuron_index*ntrials+itrial] = sp_trial_indices_temp[threadIdx.x*ntrials+itrial];
                num_coincidences[neuron_index*ntrials+itrial] = ncoinc[threadIdx.x*ntrials+itrial];
                last_spike_allowed_arr[neuron_index*ntrials+itrial] = last_spike_allowed[threadIdx.x*ntrials+itrial];
                next_spike_allowed_arr[neuron_index*ntrials+itrial] = next_spike_allowed[threadIdx.x*ntrials+itrial];
                last_spike_time_arr[neuron_index*ntrials+itrial]=last_spike_time[threadIdx.x*ntrials+itrial];
                next_spike_time_arr[neuron_index*ntrials+itrial]=next_spike_time[threadIdx.x*ntrials+itrial];
                }
            spikecount[neuron_index] = nspikes;
            current_level_index[neuron_index] = level_ind;
            __syncthreads();
                """
   
            
            return code
    else:
        def get_cuda_code(self):
            code = {}
            
            #  DECLARATION
            code['%CRITERION_DECLARE%'] = """
        int *spikecount,          // Number of spikes produced by each neuron
        int *num_coincidences,    // Count of coincidences for each neuron
            """
            
            if self.algorithm == 'exclusive':
                code['%CRITERION_DECLARE%'] += """ 
        int *sp_trial_indices,   // 
        int *last_spike_time_arr,  // pass zero array
        int *next_spike_time_arr,
        bool *last_spike_allowed_arr,
        bool *next_spike_allowed_arr,
               """
    
            
            # INITIALIZATION
            code['%CRITERION_INIT%'] = """
         int ntrials=%i;//*ntrial;
       int nspikes = spikecount[neuron_index];   
        
            """%(self.ntrials)
            
            # TIMESTEP
            code['%CRITERION_TIMESTEP%'] = """
            const int Tspike = T+spikedelay;
            """
            
            if self.algorithm == 'exclusive':
                code['%CRITERION_TIMESTEP%'] += """
           for(int itrial=0;itrial<ntrials;itrial++)
            {
                bool near_last_spike = last_spike_time_arr[neuron_index*ntrials+itrial]+%d>=Tspike;
                bool near_next_spike = next_spike_time_arr[neuron_index*ntrials+itrial]-%d<=Tspike;
                near_last_spike = near_last_spike && has_spiked;
                near_next_spike = near_next_spike && has_spiked;
    
                num_coincidences[neuron_index*ntrials+itrial] +=(near_last_spike&&last_spike_allowed_arr[neuron_index*ntrials+itrial]) || (near_next_spike&&next_spike_allowed_arr[neuron_index*ntrials+itrial]);
                bool near_both_allowed = (near_last_spike&&last_spike_allowed_arr[neuron_index*ntrials+itrial]) && (near_next_spike&&next_spike_allowed_arr[neuron_index*ntrials+itrial]);
                last_spike_allowed_arr[neuron_index*ntrials+itrial] = last_spike_allowed_arr[neuron_index*ntrials+itrial] && !near_last_spike;
                next_spike_allowed_arr[neuron_index*ntrials+itrial] = (next_spike_allowed_arr[neuron_index*ntrials+itrial] && !near_next_spike) || near_both_allowed;
        
                if(Tspike>=next_spike_time_arr[neuron_index*ntrials+itrial]){
                    sp_trial_indices[neuron_index*ntrials+itrial]+=1;
                    last_spike_time_arr[neuron_index*ntrials+itrial] = next_spike_time_arr[neuron_index*ntrials+itrial];  
                    next_spike_time_arr[neuron_index*ntrials+itrial] = spiketimes[sp_trial_indices[neuron_index*ntrials+itrial]+1];
                }
                last_spike_allowed_arr[neuron_index*ntrials+itrial] = next_spike_allowed_arr[neuron_index*ntrials+itrial];
                next_spike_allowed_arr[neuron_index*ntrials+itrial] = true;
     
            }
            nspikes += has_spiked*(T>=onset);
           
            """% (self.delta, self.delta)
            
            # FINALIZATION
            code['%CRITERION_END%'] = """
           spikecount[neuron_index] = nspikes;
            """
            
            return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.spike_count_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
        self.coincidences_gpu = gpuarray.to_gpu(zeros(self.N*self.ntrials, dtype=int32))
        self.sp_trial_indices_gpu = gpuarray.to_gpu(array(self.trials_offset,dtype=int32))
        self.last_spike_time_gpu = gpuarray.to_gpu(zeros(self.N*self.ntrials, dtype=int32))
        self.next_spike_time_gpu = gpuarray.to_gpu(zeros(self.N*self.ntrials, dtype=int32))
        self.last_spike_allowed_arr = gpuarray.to_gpu(zeros(self.N*self.ntrials, dtype=bool))
        self.next_spike_allowed_arr = gpuarray.to_gpu(ones(self.N*self.ntrials, dtype=bool))
        self.spike_count_level_gpu = gpuarray.to_gpu(zeros(self.N*self.nlevels, dtype=int32))
        self.current_level_index_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))

    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.spike_count_gpu, self.coincidences_gpu,self.sp_trial_indices_gpu,self.last_spike_time_gpu,self.next_spike_time_gpu]
        args += [self.last_spike_allowed_arr,self.next_spike_allowed_arr,self.spike_count_level_gpu,self.current_level_index_gpu]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self.coincidences = self.coincidences_gpu.get()
        self.spike_count = self.spike_count_gpu.get()
        self.spike_count_level = self.spike_count_level_gpu.get()
#        print self.sp_trial_indices_gpu.get()
    
    def get_values(self):
        return (self.coincidences, self.spike_count)
    
    def normalize(self, values):
        coincidence_count = values[0]
        spike_count = values[1]
        spike_count_temp = repeat(spike_count,self.ntrials)
#        self.target_count_temp = tile(self.target_count,self.N)
        delta = self.delta*self.dt

        if self.ntrials>1:
            coincidence_count = sum(reshape(coincidence_count,(self.N,self.ntrials)),axis=1)

        gamma = get_gamma_factor(coincidence_count, spike_count*self.ntrials, 
                                 sum(self.target_count), array(mean(self.target_rate)), 
                                 delta)
        self.spike_count_level=reshape(self.spike_count_level,(self.N,self.nlevels))
#        print self.target_count_level.shape,sum(self.target_count_level,axis=1)
#        print self.spike_count_level-sum(self.target_count_level,axis=1)
#        print abs(self.spike_count_level*self.ntrials-sum(self.target_count_level,axis=1))/sum(self.target_count_level,axis=1)
#        print self.target_count_level
        if self.fr_weight!=0:
            gamma = gamma - self.fr_weight*mean(abs(self.spike_count_level*self.ntrials-sum(self.target_count_level,axis=1))/sum(self.target_count_level,axis=1),axis=1)
        return gamma

class GammaFactor2(CriterionStruct):
    def __init__(self, delta = 4*ms,fr_weight=0, coincidence_count_algorithm = 'exclusive',nlevels=1,level_duration=100):
        self.type = 'spikes'
        self.delta = delta
        self.nlevels = nlevels
        self.fr_weight = fr_weight
        self.level_duration=level_duration
        self.coincidence_count_algorithm = coincidence_count_algorithm


class GammaFactorCriterion(Criterion):
    """
    Coincidence counter class.
    
    Counts the number of coincidences between the spikes of the neurons in the network (model spikes),
    and some user-specified data spike trains (target spikes). This number is defined as the number of 
    target spikes such that there is at least one model spike within +- ``delta``, where ``delta``
    is the half-width of the time window.
    
    Initialised as::
    
        cc = CoincidenceCounter(source, data, delta = 4*ms)
    
    with the following arguments:
    
    ``source``
        A :class:`NeuronGroup` object which neurons are being monitored.
    
    ``data``
        The list of spike times. Several spike trains can be passed in the following way.
        Define a single 1D array ``data`` which contains all the target spike times one after the
        other. Now define an array ``spiketimes_offset`` of integers so that neuron ``i`` should 
        be linked to target train: ``data[spiketimes_offset[i]], data[spiketimes_offset[i]+1]``, etc.
        
        It is essential that each spike train with the spiketimes array should begin with a spike at a
        large negative time (e.g. -1*second) and end with a spike that is a long time
        after the duration of the run (e.g. duration+1*second).
    
    ``delta=4*ms``
        The half-width of the time window for the coincidence counting algorithm.
    
    ``spiketimes_offset``
        A 1D array, ``spiketimes_offset[i]`` is the index of the first spike of 
        the target train associated to neuron i.
        
    ``spikedelays``
        A 1D array with spike delays for each neuron. All spikes from the target 
        train associated to neuron i are shifted by ``spikedelays[i]``.
        
    ``coincidence_count_algorithm``
        If set to ``'exclusive'``, the algorithm cannot count more than one
        coincidence for each model spike.
        If set to ``'inclusive'``, the algorithm can count several coincidences
        for a single model spike.
    
    ``onset``
        A scalar value in seconds giving the start of the counting: no
        coincidences are counted before ``onset``.
    
    Has three attributes:
    
    ``coincidences``
        The number of coincidences for each neuron of the :class:`NeuronGroup`.
        ``coincidences[i]`` is the number oflength[i]`` is the spike
  count for n i.
        
    ``target_length``
        The number of spikes in the target spike train associated to each neuron.
    """
    type = 'spikes'
    def initialize(self, delta=4*ms, fr_weight=0,coincidence_count_algorithm='exclusive'):
        self.algorithm = coincidence_count_algorithm
        self.delta = int(rint(delta / self.dt))
        self.fr_weight = fr_weight
        self.spike_count = zeros(self.N, dtype='int')
        self.coincidences = zeros(self.N, dtype='int')
        self.spiketime_index = self.spikes_offset
        self.last_spike_time = array(rint(self.spikes_inline[self.spiketime_index] / self.dt), dtype=int)
        self.next_spike_time = array(rint(self.spikes_inline[self.spiketime_index + 1] / self.dt), dtype=int)

        # First target spikes (needed for the computation of 
        #   the target train firing rates)
#        self.first_target_spike = zeros(self.N)

        self.last_spike_allowed = ones(self.N, dtype='bool')
        self.next_spike_allowed = ones(self.N, dtype='bool')
        
    def spike_call(self, spiking_neurons):
        dt = self.dt
        t = self.step()*dt
        
        spiking_neurons = array(spiking_neurons)
        if len(spiking_neurons)>0:
            
            if t >= self.onset:
                self.spike_count[spiking_neurons] += 1

            T_spiking = array(rint((t + self.delays[spiking_neurons]) / dt), dtype=int)

            remaining_neurons = spiking_neurons
            remaining_T_spiking = T_spiking
            while True:
                remaining_indices, = (remaining_T_spiking > self.next_spike_time[remaining_neurons]).nonzero()
                if len(remaining_indices):
                    indices = remaining_neurons[remaining_indices]
                    self.spiketime_index[indices] += 1
                    self.last_spike_time[indices] = self.next_spike_time[indices]
                    self.next_spike_time[indices] = array(rint(self.spikes_inline[self.spiketime_index[indices] + 1] / dt), dtype=int)
                    if self.algorithm == 'exclusive':
                        self.last_spike_allowed[indices] = self.next_spike_allowed[indices]
                        self.next_spike_allowed[indices] = True
                    remaining_neurons = remaining_neurons[remaining_indices]
                    remaining_T_spiking = remaining_T_spiking[remaining_indices]
                else:
                    break

            # Updates coincidences count
            near_last_spike = self.last_spike_time[spiking_neurons] + self.delta >= T_spiking
            near_next_spike = self.next_spike_time[spiking_neurons] - self.delta <= T_spiking
            last_spike_allowed = self.last_spike_allowed[spiking_neurons]
            next_spike_allowed = self.next_spike_allowed[spiking_neurons]
            I = (near_last_spike & last_spike_allowed) | (near_next_spike & next_spike_allowed)

            if t >= self.onset:
                self.coincidences[spiking_neurons[I]] += 1

            if self.algorithm == 'exclusive':
                near_both_allowed = (near_last_spike & last_spike_allowed) & (near_next_spike & next_spike_allowed)
                self.last_spike_allowed[spiking_neurons] = last_spike_allowed & -near_last_spike
                self.next_spike_allowed[spiking_neurons] = (next_spike_allowed & -near_next_spike) | near_both_allowed
    
    def get_cuda_code(self):
        code = {}
        
        #  DECLARATION
        code['%CRITERION_DECLARE%'] = """
    int *spikecount,          // Number of spikes produced by each neuron
    int *num_coincidences,    // Count of coincidences for each neuron
        """
        
        if self.algorithm == 'exclusive':
            code['%CRITERION_DECLARE%'] += """ 
    bool *last_spike_allowed_arr,
    bool *next_spike_allowed_arr,
           """
        
        # INITIALIZATION
        code['%CRITERION_INIT%'] = """
    int ncoinc = num_coincidences[neuron_index];
    int nspikes = spikecount[neuron_index];
    int last_spike_time = spiketimes[spiketime_index];
    int next_spike_time = spiketimes[spiketime_index+1];
        """
        
        if self.algorithm == 'exclusive':
            code['%CRITERION_INIT%'] += """
    bool last_spike_allowed = last_spike_allowed_arr[neuron_index];
    bool next_spike_allowed = next_spike_allowed_arr[neuron_index];
            """
        
        # TIMESTEP
        code['%CRITERION_TIMESTEP%'] = """
        const int Tspike = T+spikedelay;
        """
        
        if self.algorithm == 'inclusive':
            code['%CRITERION_TIMESTEP%'] += """ 
    ncoinc += has_spiked && (((last_spike_time+%d)>=Tspike) || ((next_spike_time-%d)<=Tspike));
            """ % (self.delta, self.delta)
        if self.algorithm == 'exclusive':
            code['%CRITERION_TIMESTEP%'] += """
        bool near_last_spike = last_spike_time+%d>=Tspike;
        bool near_next_spike = next_spike_time-%d<=Tspike;
        near_last_spike = near_last_spike && has_spiked;
        near_next_spike = near_next_spike && has_spiked;
        ncoinc += (near_last_spike&&last_spike_allowed) || (near_next_spike&&next_spike_allowed);
        bool near_both_allowed = (near_last_spike&&last_spike_allowed) && (near_next_spike&&next_spike_allowed);
        last_spike_allowed = last_spike_allowed && !near_last_spike;
        next_spike_allowed = (next_spike_allowed && !near_next_spike) || near_both_allowed;
            """ % (self.delta, self.delta)
    
        code['%CRITERION_TIMESTEP%'] += """
        nspikes += has_spiked*(T>=onset);
        if(Tspike>=next_spike_time){
            spiketime_index++;
            last_spike_time = next_spike_time;
            next_spike_time = spiketimes[spiketime_index+1];
        """
    
        if self.algorithm == 'exclusive':
            code['%CRITERION_TIMESTEP%'] += """
        last_spike_allowed = next_spike_allowed;
        next_spike_allowed = true;
            """
    
        code['%CRITERION_TIMESTEP%'] += """
        }
        """
        
        # FINALIZATION
        code['%CRITERION_END%'] = """
    num_coincidences[neuron_index] = ncoinc;
    spikecount[neuron_index] = nspikes;
        """
        
        if self.algorithm == 'exclusive':
            code['%CRITERION_END%'] += """
    last_spike_allowed_arr[neuron_index] = last_spike_allowed;
    next_spike_allowed_arr[neuron_index] = next_spike_allowed;
            """
        
        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.spike_count_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
        self.coincidences_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
        
        if self.algorithm == 'exclusive':
            self.last_spike_allowed_arr = gpuarray.to_gpu(zeros(self.N, dtype=bool))
            self.next_spike_allowed_arr = gpuarray.to_gpu(ones(self.N, dtype=bool))
    
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.spike_count_gpu, self.coincidences_gpu]
        
        if self.algorithm == 'exclusive':
            args += [self.last_spike_allowed_arr,
                     self.next_spike_allowed_arr]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self.coincidences = self.coincidences_gpu.get()
        self.spike_count = self.spike_count_gpu.get()
    
    def get_values(self):
        return (self.coincidences, self.spike_count)
    
    def normalize(self, values):
        coincidence_count = values[0]
        spike_count = values[1]
        delta = self.delta*self.dt

        gamma = get_gamma_factor(coincidence_count, spike_count, 
                                 self.target_spikes_count, self.target_spikes_rate, 
                                 delta)

        gamma = gamma - self.fr_weight*abs(spike_count-self.target_spikes_count)/self.target_spikes_count
#        print 'fr',abs(spike_count-self.target_spikes_count)/self.target_spikes_count,'fr'
        return gamma

class GammaFactor(CriterionStruct):
    def __init__(self, delta = 4*ms,fr_weight=0, coincidence_count_algorithm = 'exclusive'):
        self.type = 'spikes'
        self.delta = delta
        self.fr_weight = fr_weight
        self.coincidence_count_algorithm = coincidence_count_algorithm




class VanRossumCriterion(Criterion):
    type = 'spikes'
    def initialize(self, tau):
        self.delay_range =max(self.delays)- min(self.delays)#delay range
        self.min_delay = abs(min(self.delays))#minimum of possible delay
        self.distance_vector=zeros(self.N) 
        self.nbr_neurons_group = self.N/self.K
        
        eqs="""
        dv/dt=(-v)/tau: volt
        """
        # network to convolve target spikes with the kernel
        self.input_target=SpikeGeneratorGroup(self.K,self.spikes,clock=self.group.clock)
        self.kernel_target=NeuronGroup(self.K,model=eqs,clock=self.group.clock)
        self.C_target = DelayConnection(self.input_target, self.kernel_target, 'v', structure='dense',  max_delay=self.min_delay)     
        self.C_target.connect_one_to_one(self.input_target,self.kernel_target)
        self.C_target.delay = self.min_delay*ones_like(self.C_target.delay)

        # network to convolve population spikes with the kernel
        self.kernel_population=NeuronGroup(self.N,model=eqs,clock=self.group.clock)
        self.C_population = DelayConnection(self.group, self.kernel_population, 'v', structure='sparse',  max_delay=self.delay_range)
        for iN in xrange(self.N):
            self.C_population.delay[iN,iN] = diagflat(self.min_delay + self.delays[iN])
        self.C_population.connect_one_to_one(self.group,self.kernel_population)
        self.spikecount_mon = SpikeCounter(self.group)
        self.contained_objects = [self.kernel_population,self.C_population,self.spikecount_mon,self.input_target,self.C_target,self.kernel_target]  
        self.dt=self.group.clock.dt
        self.tau = tau

    def timestep_call(self):
        trace_population = self.kernel_population.state_('v')
        trace_target = self.kernel_target.state_('v')
        for igroup in xrange(self.K):
            self.distance_vector[igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group] += (trace_population[igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group]-trace_target[igroup])**2
        self.spikecount=self.spikecount_mon.count
    def get_values(self):
        return (self.distance_vector)
    
    def normalize(self, distance_vector):
        distance_vector[nonzero(self.spikecount==0)] = inf
        #distance_vector[nonzero(self.spikecount==1)] = inf
        return 1-distance_vector*self.group.clock.dt

    def get_cuda_code(self):
        code = {}
        
        #  DECLARATION
        code['%CRITERION_DECLARE%'] = """
    double *error_arr,
    int *count_arr,
        """
        
        # INITIALIZATION
        code['%CRITERION_INIT%'] = """
    double error = error_arr[neuron_index];
    double V_kernel =0;
    double V_target =0;
    int count=count_arr[neuron_index];
    int next_spike_time = spiketimes[spiketime_index+1];
        """
        
        # TIMESTEP
#        code['%CRITERION_TIMESTEP%'] = """
#        """
        code['%CRITERION_TIMESTEP%'] = """
        const int Tspike = T+spikedelay;
        
        if(has_spiked){       // If the neuron has spiked
            V_kernel+=1;
        }
        
        count += has_spiked*(T>=onset);
        if(Tspike>=next_spike_time){
            V_target+=1;
            spiketime_index++;
            next_spike_time = spiketimes[spiketime_index+1]; }
            

        V_kernel=V_kernel*exp(-%.8f/%.6f);
        V_target=V_target*exp(-%.8f/%.6f);
        error+=pow(V_kernel-V_target,2);
        
        
        """ % (self.dt,self.tau,self.dt,self.tau)
        
        # FINALIZATION
        code['%CRITERION_END%'] = """
    error_arr[neuron_index] = error;
    count_arr[neuron_index] = count;
        """
        
        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.error_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
        self.count_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
    
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.error_gpu,self.count_gpu]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self.distance_vector = self.error_gpu.get()
        self.spikecount = self.count_gpu.get()

        
class VanRossum(CriterionStruct):
    def __init__(self, tau):
        self.type = 'spikes'
        self.tau = tau


class BretteCriterion(Criterion):
    type = 'spikes'
    def initialize(self,tau_metric):
#        tau_metric=tau_metric**
        self.delay_range =max(self.delays)- min(self.delays)#delay range
        self.min_delay = abs(min(self.delays))#minimum of possible delay
        self.corr_vector=zeros(self.N) 
        self.norm_pop = zeros(self.N) 
        self.norm_target = zeros(self.N) 
        self.nbr_neurons_group = self.N/self.K
        
        eqs="""
        tau:second
        dv/dt=(-v)/tau: volt
        """
        # network to convolve target spikes with the kernel
        self.input_target=SpikeGeneratorGroup(self.K,self.spikes,clock=self.group.clock)
        self.kernel_target=NeuronGroup(self.N,model=eqs,clock=self.group.clock)
        self.C_target = DelayConnection(self.input_target, self.kernel_target, 'v', structure='sparse',  max_delay=self.min_delay)  
        self.kernel_target.tau=tau_metric

        for igroup in xrange(self.K):
            self.C_target.W[igroup,igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group] = ones(self.nbr_neurons_group)
            self.C_target.delay[igroup,igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group] =  self.min_delay * ones(self.nbr_neurons_group)
            
        # network to convolve population spikes with the kernel
        self.kernel_population=NeuronGroup(self.N,model=eqs,clock=self.group.clock)
        self.C_population = DelayConnection(self.group, self.kernel_population, 'v', structure='sparse',  max_delay=self.delay_range)
        for iN in xrange(self.N):
            self.C_population.delay[iN,iN] = diagflat(self.min_delay + self.delays[iN])
        self.C_population.connect_one_to_one(self.group,self.kernel_population)
        self.kernel_population.tau=tau_metric
        self.spikecount_mon = SpikeCounter(self.group)
        self.contained_objects = [self.kernel_population,self.C_population,self.spikecount_mon,self.input_target,self.C_target,self.kernel_target]  
        self.tau_metric=tau_metric
        self.dt=self.group.clock.dt
        
    def timestep_call(self):
        trace_population = self.kernel_population.state_('v')
        trace_target = self.kernel_target.state_('v')
        self.corr_vector += trace_population*trace_target
        self.norm_pop += trace_population**2
        self.norm_target += trace_target**2
        self.spikecount=self.spikecount_mon.count
        
    def get_values(self):
        #print self.corr_vector,self.norm_pop,self.norm_target
        return (self.corr_vector,self.norm_pop,self.norm_target)
    
    def normalize(self, values):
        corr_vector=values[0]
        norm_pop=values[1]
        norm_target=values[2]
        corr_vector[nonzero(self.spikecount==0)] = -inf
        temp=self.corr_vector/self.tau_metric/sqrt(norm_pop)/sqrt(norm_target)
        temp[isnan(temp)] = -inf
        return temp

    def get_cuda_code(self):
        code = {}
        
        #  DECLARATION
        code['%CRITERION_DECLARE%'] = """
    double *error_arr,
    int *count_arr,
    double *tau_metric_arr,
    double *norm_pop_arr,
    double *norm_target_arr, 
        """

        
        # INITIALIZATION
        code['%CRITERION_INIT%'] = """
    double error = error_arr[neuron_index];
    double V_kernel =0;
    double V_target =0;
    double tau_metric=tau_metric_arr[neuron_index];
    int count=count_arr[neuron_index];
    int next_spike_time = spiketimes[spiketime_index+1];
    double norm_pop=norm_pop_arr[neuron_index];
    double norm_target=norm_target_arr[neuron_index];
        """
        
        # TIMESTEP
#        code['%CRITERION_TIMESTEP%'] = """
#        """
        code['%CRITERION_TIMESTEP%'] = """
        const int Tspike = T+spikedelay;
        
        if(has_spiked){       // If the neuron has spiked
            V_kernel+=1;
        }
        
        count += has_spiked*(T>=onset);
        if(Tspike>=next_spike_time){
            V_target+=1;
            spiketime_index++;
            next_spike_time = spiketimes[spiketime_index+1]; }
        
        V_kernel=V_kernel*exp(-%.8f/tau_metric);
        V_target=V_target*exp(-%.8f/tau_metric);
        
        error+=V_kernel*V_target;
        norm_pop+=pow(V_kernel,2);
        norm_target +=pow(V_target,2);
        
        
        """ % (self.dt,self.dt)
        
        # FINALIZATION
        code['%CRITERION_END%'] = """
    error_arr[neuron_index] = error;
    count_arr[neuron_index] = count;
    norm_pop_arr[neuron_index] = norm_pop;
    norm_target_arr[neuron_index] = norm_target;
        """
        
        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.error_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
        self.count_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
        self.tau_metric_gpu = gpuarray.to_gpu(array(self.tau_metric, dtype=double))
        self.norm_pop_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
        self.norm_target_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
        
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.error_gpu,self.count_gpu,self.tau_metric_gpu,self.norm_pop_gpu,self.norm_target_gpu]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self.corr_vector = self.error_gpu.get()
        self.spikecount = self.count_gpu.get()
        self.norm_pop = self.norm_pop_gpu.get()
        self.norm_target = self.norm_target_gpu.get()

class Brette(CriterionStruct):
    def __init__(self):
        self.type = 'spikes'



from brian import *
from time import time

N = 1 #number of neurons

# Parameters
C=281*pF
gL=30*nS
taum=C/gL
EL=-70.6*mV
VT=-50.4*mV
DeltaT=2*mV
Vcut=VT+5*DeltaT

b=0*nA
a=3*gL
tauw=2*taum
Vr=-70.6*mV

#For the sinusoidal stim current
ampl = 1.0*nA#*nS*mV
f=10/second#*Hz

eqs="""
dvm/dt=(gL*(EL-vm)+gL*DeltaT*exp((vm-VT)/DeltaT)+I-w + ampl*sin(2*pi*f*t))/C : volt
dw/dt=(a*(vm-EL)-w)/tauw : amp
I : amp
"""

neuron=NeuronGroup(N,model=eqs,threshold=Vcut,reset="vm=Vr;w+=b",freeze=True)
neuron.vm=EL
trace=StateMonitor(neuron,'vm',record=0)
spikes=SpikeMonitor(neuron)

run(100*ms)

plot(trace.times/ms,trace[0]/mV)
show()



"""
Real-time Brian

See BEP-20-AER interface
"""
from brian.network import NetworkOperation
from brian.clock import EventClock
from brian.stdunits import ms
from time import sleep,time

__all__=['RealtimeController']

class RealtimeController(NetworkOperation):
    '''
    Pauses are inserted so that Brian runs always between real time and
    about 50 ms ahead of it (probably a bit more).
    Principle: every 50 ms, so we insert a sleep() to synchronize.
    
    Typical use::
      R=RealtimeController()
      run(1*second)
      R.reinit() # will resynchronise real time at next run
      run(2*second)
    '''
    def __init__(self,dt=50*ms,verbose=False):
        '''
        dt is the resynchronisation period
        '''
        self.clock = EventClock(dt=dt) # this could be a global preference
        self.when = 'end'
        self.first_time=True
        self.verbose=verbose # for debugging

    def __call__(self):
        # First time: synchronise Brian and real time
        if self.first_time: # stores the start time
            self.synchronise()
            self.first_time=False
        real_time=time()+self.offset
        if self.verbose:
            print self.clock.t,real_time
            if self.clock._t<real_time:
                print 'Running late'
        if self.clock._t>real_time: # synchronize real time and clock
            sleep(self.clock._t-real_time)
            
    def synchronise(self):
        '''
        Sets the offset to the real clock so that Brian and real time are
        synchronised.
        '''
        self.offset=self.clock._t-time()
        
    def reinit(self):
        '''
        The clock synchronise with real time at the next call.
        '''
        self.first_time=True

if __name__=='__main__':
    from brian import *
    
    R=RealtimeController(dt=100*ms,verbose=True)
    run(3*second)
    sleep(1*second)
    R.reinit() # try to comment it!
    run(2*second)
    



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.33
#
# Don't modify this file, modify the SWIG interface instead.
# This file is compatible with both classic and new-style classes.

import _test
import new
new_instancemethod = new.instancemethod
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'PySwigObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method: return method(self, value)
    if (not static) or hasattr(self, name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)

def _swig_getattr(self, class_type, name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method: return method(self)
    raise AttributeError, name

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

import types
try:
    _object = types.ObjectType
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0
del types


fact = _test.fact
my_mod = _test.my_mod
get_time = _test.get_time

cvar = _test.cvar




'''
Runs all of the examples in the examples/ folder.

This version clears any history of which examples did or didn't pass.
Better to use allexamples_cumulative but delete the examples_completed.txt
file first.
'''

open('examples_completed.txt', 'w').write('')
import allexamples_cumulative



"""
Lp electrode compensation example.
"""
from brian import *
from brian.library.electrophysiology import Lp_compensate
import numpy

# load the injected current and the raw voltage trace of a neuron
# in response to this current. The two variables are 1D vectors of 
# equal length.
I = numpy.load("current.npy")
Vraw = numpy.load("trace.npy")

# launch the compensation procedure. The sampling period must be specified
# (here, 10 kHz). This function returns the compensated trace, and the
# corresponding neuron+electrode parameters: R, tau, Vr, Re, taue.
Vcomp, params = Lp_compensate(I, Vraw, .1*ms)

print "Compensation terminated, Re=%.1f MOhm, taue=%.2f ms" % \
    (params[3,0]/Mohm, params[4,0]/ms)

subplot(211)
plot(I, 'k')

subplot(212)
plot(Vraw, 'k')
plot(Vcomp, 'r')

show()



from brian import *
from matplotlib import rc
rc('font', **{'family':'sans-serif', 'sans-serif':['Helvetica']})
rc('text', usetex=True)

#Parameters from Kuznetsov et al. (2006) Table 1
Mg = 1.4 * mmole
F = 96485.3999 * coulomb / mole
E_Ca = 100 * mV
g_L = 0.05 * msiemens / cm2
g_maxCa = 0.2 * msiemens / cm2
g_maxK = 0.4 * msiemens / cm2
g_maxKCa = 0.3 * msiemens / cm2
g_AMPA = 0.07 * msiemens / cm2 #From Figure 5 Kuznetsov et al. (2006)
g_maxNMDA = 1 * msiemens / cm2 #From Figure 5 Kuznetsov et al. (2006)
E_k = -90 * mV
E_L = -50 * mV
E_AMPA = 0 * mV
E_NMDA = 0 * mV
p_Ca = 2500 * um / second
beta = 0.05 # ratio of free to total calcium
z = 2 # Valence of Calcium Ion
C = 1 * uF / cm2
k = 250 * nmole / cm2
r = 1 * um
eqs = Equations('''
    dV/dt = (g_Ca*(E_Ca-V) + g_K *(E_k-V) + g_KCa*(E_k-V) + g_L*(E_L-V) + g_NMDA*(E_NMDA-V)+g_AMPA*(E_AMPA-V)+ I)/C: volt
    dCa/dt = (4*beta)*(g_Ca/(z*F)*(E_Ca-V) - p_Ca*Ca/r) : nmole/cm2
    g_KCa = g_maxKCa * Ca**4/(Ca**4 + k**4) : msiemens/cm2
    alpha_Ca = -0.0032*(V+50*mV)/((exp(-(V+50*mV)/(5*mV))-1)*mV) : 1
    beta_Ca = 0.05*exp(-(V+55*mV)/(40*mV)) : 1
    g_Ca = g_maxCa*(alpha_Ca/(alpha_Ca + beta_Ca))**4 : usiemens/cm2
    g_NMDA = g_maxNMDA/(1+Mg/(10*mmole)*exp(-V/(12.5*mV))) : usiemens/cm2
    g_K = g_maxK/(1+exp(-(V+10*mV)/(7*mV)))    : usiemens/cm2
    I : uA/cm2

    ''')
test = NeuronGroup(1, model=eqs, threshold= -40 * mV, reset= -53 * mV)
trace = StateMonitor(test, 'V', record=True)
stimulus = StateMonitor(test, 'I', record=True)
test.V = -50 * mV
run(1000 * msecond)
test.I = 1 * uA / cm ** 2
run(500 * msecond)
test.I = 0 * uA / cm ** 2
run(500 * msecond)

figure()
title('Simulation')
subplot(211)
plot(trace.times / ms, trace[0] / mV)
ylabel('Voltage (mV)')
subplot(212)
plot(stimulus.times / ms, stimulus[0] / uA * cm2)
ylabel(r'\Current Density ($\frac{\mu A}{cm^{2}}$)')
xlabel('Time (ms)')
show()




# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Membrane equations for Brian models.
'''
__all__ = ['Current', 'IonicCurrent', 'InjectedCurrent', 'MembraneEquation']

from equations import *
from units import have_same_dimensions, get_unit, second, volt, amp
from warnings import warn


class Current(Equations):
    '''
    A set of equations defining a current.
    
    current_name is the name of the variable that must be added as a membrane current
    to the membrane equation.
    '''
    def __init__(self, expr='', current_name=None, level=0, surfacic=False, **kwd):
        Equations.__init__(self, expr, level=level + 1, **kwd)
        if surfacic: # A surfacic current is multiplied by membrane area in a MembraneEquation
            self._prefix = '__scurrent_'
        else:
            self._prefix = '__current_'
        # Find which variable is the current
        if current_name: # Explicitly given
            self.set_current_name(current_name)
        elif expr != '': # Guess
            if len(self._units) == 2: # only one variable (the other one is t)
                # Only 1 variable: it's the current
                correct_names = [name for name in self._units.keys() if name != 't']
                if len(correct_names) != 1:
                    raise NameError, "The equations do not include time (variable t)"
                name, = correct_names
                self.set_current_name(name)
            else:
                # Look for variables with dimensions of current: won't work with units off!
                current_names = [name for name, unit in self._units.iteritems()\
                               if have_same_dimensions(unit, amp)]
                if len(current_names) == 1: # only one current
                    self.set_current_name(current_names[0])
                else:
                    warn("The current variable could not be found!")

    def set_current_name(self, name):
        if name != 't':
            if name is None:
                name = unique_id()
            current_name = self._prefix + name
            self.add_eq(current_name, name, self._units[name]) # not an alias because read-only

    def __iadd__(self, other):
        # Adding a MembraneEquation
        if isinstance(other, MembraneEquation):
            return other.__iadd__(self)
        else:
            return Equations.__iadd__(self, other)


class IonicCurrent(Current):
    '''
    A ionic current; current direction is defined from intracellular
    to extracellular.
    '''
    def set_current_name(self, name):
        if name != 't':
            if name is None:
                name = unique_id()
            current_name = self._prefix + name
            self.add_eq(current_name, '-' + name, self._units[name])

InjectedCurrent = Current


class MembraneEquation(Equations):
    '''
    A membrane equation, defined by a capacitance C and a sum of currents.
    Ex:
      eq=MembraneEquation(200*pF)
      eq=MembraneEquation(200*pF,vm='V')
    No more than one membrane equation allowed per system of equations.
    '''
    def __init__(self, C=None, **kwd):
        if C is not None:
            expr = '''
            dvm/dt=__membrane_Im/C : volt
            __membrane_Im=0*unit : unit
            '''
            Equations.__init__(self, expr, C=C, unit=get_unit(C * volt / second), **kwd)
        else:
            Equations.__init__(self)

    def __iadd__(self, other):
        Equations.__iadd__(self, other)
        self.set_membrane_current()
        return self

    def set_membrane_current(self):
        current_vars = [name for name in self._eq_names if name.startswith('__current_')] # point current
        scurrent_vars = [name for name in self._eq_names if name.startswith('__scurrent_')] # surfacic current
        if scurrent_vars != []:
            if current_vars != []:
                self._string['__membrane_Im'] = '+'.join(current_vars) + '+__area*(' + '+'.join(scurrent_vars) + ')'
            else:
                self._string['__membrane_Im'] = '__area*(' + '+'.join(scurrent_vars) + ')'
            self._namespace[name]['__area'] = self.area
        elif current_vars != []:
            self._string['__membrane_Im'] = '+'.join(current_vars)
